@@ -24,114 +24,92 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Any  # pylint: disable=unused-import
-from typing import Dict  # pylint: disable=unused-import
-from typing import List  # pylint: disable=unused-import
-from typing import Optional  # pylint: disable=unused-import
+from __future__ import annotations
+
+import typing
+from typing import Any
+from typing import NamedTuple
+from typing import Optional
 from typing import cast
 
+import gc
 import os
 import sys
 import logging
-import uuid
-from collections import namedtuple
+import weakref
+import pprint
 from collections import defaultdict
 
 import nbxmpp
+from nbxmpp.idlequeue import IdleQueue
 from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import GObject
 
 import gajim
+from gajim.common import types
 from gajim.common import config as c_config
 from gajim.common import configpaths
 from gajim.common import ged as ged_module
 from gajim.common.i18n import LANG
 from gajim.common.const import Display
-from gajim.common.events import Events
-from gajim.common.types import NetworkEventsControllerT  # pylint: disable=unused-import
-from gajim.common.types import InterfaceT  # pylint: disable=unused-import
-from gajim.common.types import ConnectionT  # pylint: disable=unused-import
-from gajim.common.types import LegacyContactsAPIT  # pylint: disable=unused-import
-from gajim.common.types import SettingsT  # pylint: disable=unused-import
 
-interface = cast(InterfaceT, None)
+if typing.TYPE_CHECKING:
+    from gajim.gui.main import MainWindow
+    from gajim.gui.application import GajimApplication
+    from gajim.common.storage.cache import CacheStorage
+    from gajim.common.storage.archive import MessageArchiveStorage
+    from gajim.common.cert_store import CertificateStore
+
+
+interface = cast(types.InterfaceT, None)
 thread_interface = lambda *args: None # Interface to run a thread and then a callback
 config = c_config.Config()
-settings = cast(SettingsT, None)
+settings = cast(types.SettingsT, None)
 version = gajim.__version__
-connections = {}  # type: Dict[str, ConnectionT]
-avatar_cache = {}  # type: Dict[str, Dict[str, Any]]
-bob_cache = {} # type: Dict[str, bytes]
+connections: dict[str, types.Client] = {}
+avatar_cache: dict[str, dict[str, Any]] = {}
+bob_cache: dict[str, bytes] = {}
 ipython_window = None
-app = None  # Gtk.Application
+app = None # type: GajimApplication
+window = None # type: MainWindow
 
 ged = ged_module.GlobalEventsDispatcher() # Global Events Dispatcher
-nec = cast(NetworkEventsControllerT, None)
-plugin_manager = None # Plugins Manager
+plugin_manager = cast(types.PluginManagerT, None)  # Plugins Manager
+plugin_repository = cast(types.PluginRepositoryT, None)
 
 class Storage:
     def __init__(self):
-        self.cache = None
-        self.archive = None
+        self.cache: CacheStorage = None
+        self.archive: MessageArchiveStorage = None
 
 storage = Storage()
 
-css_config = None
+css_config = cast(types.CSSConfigT, None)
 
-transport_type = {}  # type: Dict[str, str]
-
-# dict of time of the latest incoming message per jid
-# {acct1: {jid1: time1, jid2: time2}, }
-last_message_time = {}  # type: Dict[str, Dict[str, float]]
-
-contacts = cast(LegacyContactsAPIT, None)
-
-# tell if we are connected to the room or not
-# {acct: {room_jid: True}}
-gc_connected = {}  # type: Dict[str, Dict[str, bool]]
-
-# dict of the pass required to enter a room
-# {room_jid: password}
-gc_passwords = {}  # type: Dict[str, str]
+transport_type: dict[str, str] = {}
 
 # dict of rooms that must be automatically configured
 # and for which we have a list of invities
 # {account: {room_jid: {'invities': []}}}
-automatic_rooms = {}  # type: Dict[str, Dict[str, Dict[str, List[str]]]]
-
- # dict of groups, holds if they are expanded or not
-groups = {}  # type: Dict[str, Dict[str, Dict[str, bool]]]
-
-# list of contacts that has just signed in
-newly_added = {}  # type: Dict[str, List[str]]
+automatic_rooms: dict[str, dict[str, dict[str, list[str]]]] = {}
 
 # list of contacts that has just signed out
-to_be_removed = {}  # type: Dict[str, List[str]]
-
-events = Events()
+to_be_removed: dict[str, list[str]] = {}
 
 notification = None
 
 # list of our nick names in each account
-nicks = {}  # type: Dict[str, str]
-
-# should we block 'contact signed in' notifications for this account?
-# this is only for the first 30 seconds after we change our show
-# to something else than offline
-# can also contain account/transport_jid to block notifications for contacts
-# from this transport
-block_signed_in_notifications = {}  # type: Dict[str, bool]
+nicks: dict[str, str] = {}
 
 proxy65_manager = None
 
-cert_store = None
+cert_store = cast('CertificateStore', None)
 
 task_manager = None
 
-# zeroconf account name
-ZEROCONF_ACC_NAME = 'Local'
-
 # These will be set in app.gui_interface.
-idlequeue = None  # type: nbxmpp.idlequeue.IdleQueue
+idlequeue = cast(IdleQueue, None)
 socks5queue = None
 
 gupnp_igd = None
@@ -139,8 +117,6 @@ gupnp_igd = None
 gsound_ctx = None
 
 _dependencies = {
-    'AVAHI': False,
-    'PYBONJOUR': False,
     'FARSTREAM': False,
     'GST': False,
     'AV': False,
@@ -149,34 +125,45 @@ _dependencies = {
     'GSOUND': False,
     'GSPELL': False,
     'IDLE': False,
+    'APPINDICATOR': False,
+    'AYATANA_APPINDICATOR': False,
+    'SENTRY_SDK': False,
 }
 
-_tasks = defaultdict(list)  # type: Dict[int, List[Any]]
+_tasks: dict[int, list[Any]] = defaultdict(list)
 
-def print_version():
+
+class RecentGroupchat(NamedTuple):
+    room: str
+    server: str
+    nickname: str
+
+
+def print_version() -> None:
     log('gajim').info('Gajim Version: %s', gajim.__version__)
 
 
-def get_client(account):
+def get_client(account: str) -> types.Client:
     return connections[account]
 
 
-def is_installed(dependency):
-    if dependency == 'ZEROCONF':
-        # Alias for checking zeroconf libs
-        return _dependencies['AVAHI'] or _dependencies['PYBONJOUR']
+def get_clients() -> list[types.Client]:
+    return list(connections.values())
+
+
+def is_installed(dependency: str) -> bool:
     return _dependencies[dependency]
 
 
-def is_flatpak():
+def is_flatpak() -> bool:
     return gajim.IS_FLATPAK
 
 
-def is_portable():
+def is_portable() -> bool:
     return gajim.IS_PORTABLE
 
 
-def is_display(display):
+def is_display(display: Display) -> bool:
     # XWayland reports as Display X11, so try with env var
     is_wayland = os.environ.get('XDG_SESSION_TYPE') == 'wayland'
     if is_wayland and display == Display.WAYLAND:
@@ -188,30 +175,21 @@ def is_display(display):
         return False
     return default.__class__.__name__ == display.value
 
-def disable_dependency(dependency):
+
+def disable_dependency(dependency: str) -> None:
     _dependencies[dependency] = False
 
-def detect_dependencies():
+
+def detect_dependencies() -> None:
     import gi
 
-    # ZEROCONF
-    try:
-        import pybonjour  # pylint: disable=unused-import
-        _dependencies['PYBONJOUR'] = True
-    except Exception:
-        pass
-
-    try:
-        gi.require_version('Avahi', '0.6')
-        from gi.repository import Avahi  # pylint: disable=unused-import
-        _dependencies['AVAHI'] = True
-    except Exception:
-        pass
-
     try:
         gi.require_version('Gst', '1.0')
+        gi.require_version('GstPbutils', '1.0')
         from gi.repository import Gst
-        _dependencies['GST'] = True
+        from gi.repository import GstPbutils  # pylint: disable=unused-import
+        success, _argv = Gst.init_check(None)
+        _dependencies['GST'] = success
     except Exception:
         pass
 
@@ -219,17 +197,16 @@ def detect_dependencies():
         gi.require_version('Farstream', '0.2')
         from gi.repository import Farstream
         _dependencies['FARSTREAM'] = True
-    except Exception:
-        pass
+    except Exception as error:
+        log('gajim').warning('AV dependency test failed: %s', error)
 
     try:
         if _dependencies['GST'] and _dependencies['FARSTREAM']:
-            Gst.init(None)
             conference = Gst.ElementFactory.make('fsrtpconference', None)
             conference.new_session(Farstream.MediaType.AUDIO)
             from gajim.gui.gstreamer import create_gtk_widget
-            sink, _, _ = create_gtk_widget()
-            if sink is not None:
+            gtk_widget = create_gtk_widget()
+            if gtk_widget is not None:
                 _dependencies['AV'] = True
     except Exception as error:
         log('gajim').warning('AV dependency test failed: %s', error)
@@ -263,7 +240,6 @@ def detect_dependencies():
     # GSOUND
     try:
         gi.require_version('GSound', '1.0')
-        from gi.repository import GLib
         from gi.repository import GSound
         global gsound_ctx
         gsound_ctx = GSound.Context()
@@ -288,13 +264,36 @@ def detect_dependencies():
     except (ImportError, ValueError):
         pass
 
+    # APPINDICATOR
+    try:
+        gi.require_version('AppIndicator3', '0.1')
+        from gi.repository import AppIndicator3  # pylint: disable=unused-import
+        _dependencies['APPINDICATOR'] = True
+    except (ImportError, ValueError):
+        pass
+    # AYATANA APPINDICATOR
+    try:
+        gi.require_version('AyatanaAppIndicator3', '0.1')
+        from gi.repository import AyatanaAppIndicator3  # pylint: disable=unused-import
+        _dependencies['AYATANA_APPINDICATOR'] = True
+    except (ImportError, ValueError):
+        pass
+
+    # SENTRY SDK
+    try:
+        import sentry_sdk  # pylint: disable=unused-import
+        _dependencies['SENTRY_SDK'] = True
+    except ImportError:
+        pass
+
     # Print results
     for dep, val in _dependencies.items():
         log('gajim').info('%-13s %s', dep, val)
 
     log('gajim').info('Used language: %s', LANG)
 
-def detect_desktop_env():
+
+def detect_desktop_env() -> Optional[str]:
     if sys.platform in ('win32', 'darwin'):
         return sys.platform
 
@@ -308,23 +307,13 @@ def detect_desktop_env():
 
 desktop_env = detect_desktop_env()
 
-def get_an_id():
-    return str(uuid.uuid4())
 
-def get_nick_from_jid(jid):
-    pos = jid.find('@')
-    return jid[:pos]
-
-def get_server_from_jid(jid):
+def get_server_from_jid(jid: str) -> str:
     pos = jid.find('@') + 1 # after @
     return jid[pos:]
 
-def get_name_and_server_from_jid(jid):
-    name = get_nick_from_jid(jid)
-    server = get_server_from_jid(jid)
-    return name, server
 
-def get_room_and_nick_from_fjid(jid):
+def get_room_and_nick_from_fjid(jid: str) -> list[str]:
     # fake jid is the jid for a contact in a room
     # gaim@conference.jabber.no/nick/nick-continued
     # return ('gaim@conference.jabber.no', 'nick/nick-continued')
@@ -333,53 +322,22 @@ def get_room_and_nick_from_fjid(jid):
         l.append('')
     return l
 
-def get_real_jid_from_fjid(account, fjid):
-    """
-    Return real jid or returns None, if we don't know the real jid
-    """
-    room_jid, nick = get_room_and_nick_from_fjid(fjid)
-    if not nick: # It's not a fake_jid, it is a real jid
-        return fjid # we return the real jid
-    real_jid = fjid
-    if interface.msg_win_mgr.get_gc_control(room_jid, account):
-        # It's a pm, so if we have real jid it's in contact.jid
-        gc_contact = contacts.get_gc_contact(account, room_jid, nick)
-        if not gc_contact:
-            return
-        # gc_contact.jid is None when it's not a real jid (we don't know real jid)
-        real_jid = gc_contact.jid
-    return real_jid
 
-def get_room_from_fjid(jid):
-    return get_room_and_nick_from_fjid(jid)[0]
-
-def get_contact_name_from_jid(account, jid):
-    c = contacts.get_first_contact_from_jid(account, jid)
-    return c.name
-
-def get_jid_without_resource(jid):
+def get_jid_without_resource(jid: str) -> str:
     return jid.split('/')[0]
 
-def construct_fjid(room_jid, nick):
-    # fake jid is the jid for a contact in a room
-    # gaim@conference.jabber.org/nick
-    return room_jid + '/' + nick
 
-def get_resource_from_jid(jid):
-    jids = jid.split('/', 1)
-    if len(jids) > 1:
-        return jids[1] # abc@doremi.org/res/res-continued
-    return ''
-
-def get_number_of_accounts():
+def get_number_of_accounts() -> int:
     """
     Return the number of ALL accounts
     """
     return len(connections.keys())
 
-def get_number_of_connected_accounts(accounts_list=None):
+
+def get_number_of_connected_accounts(
+        accounts_list: Optional[list[str]] = None) -> int:
     """
-    Returns the number of CONNECTED accounts. Uou can optionally pass an
+    Returns the number of connected accounts. You can optionally pass an
     accounts_list and if you do those will be checked, else all will be checked
     """
     connected_accounts = 0
@@ -392,18 +350,20 @@ def get_number_of_connected_accounts(accounts_list=None):
             connected_accounts = connected_accounts + 1
     return connected_accounts
 
-def get_available_clients():
-    clients = []
+
+def get_available_clients() -> list[types.Client]:
+    clients: list[types.Client] = []
     for client in connections.values():
         if client.state.is_available:
             clients.append(client)
     return clients
 
-def get_connected_accounts(exclude_local=False):
+
+def get_connected_accounts(exclude_local: bool = False) -> list[str]:
     """
     Returns a list of CONNECTED accounts
     """
-    account_list = []
+    account_list: list[str] = []
     for account in connections:
         if account == 'Local' and exclude_local:
             continue
@@ -411,27 +371,25 @@ def get_connected_accounts(exclude_local=False):
             account_list.append(account)
     return account_list
 
-def get_accounts_sorted():
+
+def get_accounts_sorted() -> list[str]:
     '''
     Get all accounts alphabetically sorted with Local first
     '''
     account_list = settings.get_accounts()
     account_list.sort(key=str.lower)
-    if 'Local' in account_list:
-        account_list.remove('Local')
-        account_list.insert(0, 'Local')
     return account_list
 
-def get_enabled_accounts_with_labels(exclude_local=True, connected_only=False,
-                                     private_storage_only=False):
+
+def get_enabled_accounts_with_labels(
+        connected_only: bool = False,
+        private_storage_only: bool = False) -> list[list[str]]:
     """
     Returns a list with [account, account_label] entries.
     Order by account_label
     """
-    accounts = []
+    accounts: list[list[str]] = []
     for acc in connections:
-        if exclude_local and account_is_zeroconf(acc):
-            continue
         if connected_only and not account_is_connected(acc):
             continue
         if private_storage_only and not account_supports_private_storage(acc):
@@ -442,51 +400,49 @@ def get_enabled_accounts_with_labels(exclude_local=True, connected_only=False,
     accounts.sort(key=lambda xs: str.lower(xs[1]))
     return accounts
 
-def get_account_label(account):
+
+def get_account_label(account: str) -> str:
     return settings.get_account_setting(account, 'account_label') or account
 
-def account_is_zeroconf(account):
-    return connections[account].is_zeroconf
 
-def account_supports_private_storage(account):
+def account_supports_private_storage(account: str) -> bool:
     # If Delimiter module is not available we can assume
     # Private Storage is not available
     return connections[account].get_module('Delimiter').available
 
-def account_is_connected(account):
+
+def account_is_connected(account: str) -> bool:
     if account not in connections:
         return False
     return (connections[account].state.is_connected or
             connections[account].state.is_available)
 
-def account_is_available(account):
+
+def account_is_available(account: str) -> bool:
     if account not in connections:
         return False
     return connections[account].state.is_available
 
-def account_is_disconnected(account):
+
+def account_is_disconnected(account: str) -> bool:
     return not account_is_connected(account)
 
-def zeroconf_is_connected():
-    return account_is_connected(ZEROCONF_ACC_NAME) and \
-            settings.get_account_setting(ZEROCONF_ACC_NAME, 'is_zeroconf')
 
-def in_groupchat(account, room_jid):
-    room_jid = str(room_jid)
-    if room_jid not in gc_connected[account]:
-        return False
-    return gc_connected[account][room_jid]
+def get_transport_name_from_jid(
+        jid: str,
+        use_config_setting: bool = True) -> Optional[str]:
 
-def get_transport_name_from_jid(jid, use_config_setting=True):
     """
     Returns 'gg', 'irc' etc
 
     If JID is not from transport returns None.
     """
+    # TODO: Rewrite/remove
+
     #FIXME: jid can be None! one TB I saw had this problem:
     # in the code block # it is a groupchat presence in handle_event_notify
     # jid was None. Yann why?
-    if not jid or (use_config_setting and not config.get('use_transports_iconsets')):
+    if not jid:
         return
 
     host = get_server_from_jid(jid)
@@ -509,13 +465,14 @@ def get_transport_name_from_jid(jid, use_config_setting=True):
         return 'facebook'
     return None
 
-def jid_is_transport(jid):
+def jid_is_transport(jid: str) -> bool:
     # if not '@' or '@' starts the jid then it is transport
     if jid.find('@') <= 0:
         return True
     return False
 
-def get_jid_from_account(account_name):
+
+def get_jid_from_account(account_name: str) -> str:
     """
     Return the jid we use in the given account
     """
@@ -524,21 +481,14 @@ def get_jid_from_account(account_name):
     jid = name + '@' + hostname
     return jid
 
-def get_account_from_jid(jid):
+
+def get_account_from_jid(jid: str) -> Optional[str]:
     for account in settings.get_accounts():
         if jid == get_jid_from_account(account):
             return account
 
-def get_our_jids():
-    """
-    Returns a list of the jids we use in our accounts
-    """
-    our_jids = list()
-    for account in contacts.get_accounts():
-        our_jids.append(get_jid_from_account(account))
-    return our_jids
 
-def get_hostname_from_account(account_name, use_srv=False):
+def get_hostname_from_account(account_name: str, use_srv: bool = False) -> str:
     """
     Returns hostname (if custom hostname is used, that is returned)
     """
@@ -548,44 +498,32 @@ def get_hostname_from_account(account_name, use_srv=False):
         return settings.get_account_setting(account_name, 'custom_host')
     return settings.get_account_setting(account_name, 'hostname')
 
-def get_notification_image_prefix(jid):
+
+def get_notification_image_prefix(jid: str) -> str:
     """
     Returns the prefix for the notification images
     """
     transport_name = get_transport_name_from_jid(jid)
-    if transport_name in ('icq', 'facebook'):
+    if transport_name in ['icq', 'facebook']:
         prefix = transport_name
     else:
         prefix = 'jabber'
     return prefix
 
-def get_name_from_jid(account, jid):
-    """
-    Return from JID's shown name and if no contact returns jids
-    """
-    contact = contacts.get_first_contact_from_jid(account, jid)
-    if contact:
-        actor = contact.get_shown_name()
-    else:
-        actor = jid
-    return actor
 
-
-def get_recent_groupchats(account):
+def get_recent_groupchats(account: str) -> list[RecentGroupchat]:
     recent_groupchats = settings.get_account_setting(
         account, 'recent_groupchats').split()
 
-    RecentGroupchat = namedtuple('RecentGroupchat',
-                                 ['room', 'server', 'nickname'])
-
-    recent_list = []
+    recent_list: list[RecentGroupchat] = []
     for groupchat in recent_groupchats:
         jid = nbxmpp.JID.from_string(groupchat)
         recent = RecentGroupchat(jid.localpart, jid.domain, jid.resource)
         recent_list.append(recent)
     return recent_list
 
-def add_recent_groupchat(account, room_jid, nickname):
+
+def add_recent_groupchat(account: str, room_jid: str, nickname: str) -> None:
     recent = settings.get_account_setting(
         account, 'recent_groupchats').split()
     full_jid = room_jid + '/' + nickname
@@ -597,7 +535,8 @@ def add_recent_groupchat(account, room_jid, nickname):
     config_value = ' '.join(recent)
     settings.set_account_setting(account, 'recent_groupchats', config_value)
 
-def get_priority(account, show):
+
+def get_priority(account: str, show: str) -> int:
     """
     Return the priority an account must have
     """
@@ -615,23 +554,27 @@ def get_priority(account, show):
         prio = 127
     return prio
 
-def log(domain):
+
+def log(domain: str) -> logging.Logger:
     if domain != 'gajim':
         domain = 'gajim.%s' % domain
     return logging.getLogger(domain)
 
-def prefers_app_menu():
+
+def prefers_app_menu() -> bool:
     if sys.platform == 'darwin':
         return True
     if sys.platform == 'win32':
         return False
     return app.prefers_app_menu()
 
-def load_css_config():
+
+def load_css_config() -> None:
     global css_config
     from gajim.gui.css_config import CSSConfig
     css_config = CSSConfig()
 
+
 def set_debug_mode(enable: bool) -> None:
     debug_folder = configpaths.get('DEBUG')
     debug_enabled = debug_folder / 'debug-enabled'
@@ -641,11 +584,13 @@ def set_debug_mode(enable: bool) -> None:
         if debug_enabled.exists():
             debug_enabled.unlink()
 
+
 def get_debug_mode() -> bool:
     debug_folder = configpaths.get('DEBUG')
     debug_enabled = debug_folder / 'debug-enabled'
     return debug_enabled.exists()
 
+
 def get_stored_bob_data(algo_hash: str) -> Optional[bytes]:
     try:
         return bob_cache[algo_hash]
@@ -657,21 +602,12 @@ def get_stored_bob_data(algo_hash: str) -> Optional[bytes]:
             return data
     return None
 
-def get_groupchat_control(account, jid):
-    control = app.interface.msg_win_mgr.get_gc_control(jid, account)
-    if control is not None:
-        return control
-    try:
-        return app.interface.minimized_controls[account][jid]
-    except Exception:
-        return None
-
 
 def register_task(self, task):
     _tasks[id(self)].append(task)
 
 
-def remove_task(task, id_):
+def remove_task(task, id_) -> None:
     try:
         _tasks[id_].remove(task)
     except Exception:
@@ -681,7 +617,7 @@ def remove_task(task, id_):
             del _tasks[id_]
 
 
-def cancel_tasks(obj):
+def cancel_tasks(obj: Any) -> None:
     id_ = id(obj)
     if id_ not in _tasks:
         return
@@ -689,3 +625,50 @@ def cancel_tasks(obj):
     task_list = _tasks[id_]
     for task in task_list:
         task.cancel()
+
+
+def check_finalize(obj: Any) -> None:
+    if 'GAJIM_LEAK' not in os.environ:
+        return
+
+    name = obj.__class__.__name__
+    logger = logging.getLogger('gajim.leak')
+    finalizer = weakref.finalize(obj, logger.info, f'{name} has been finalized')
+
+    g_objects: list[str] = []
+    if isinstance(obj, GObject.Object):
+        g_objects.append(name)
+
+        def g_object_finalized():
+            g_objects.remove(name)
+
+        obj.weak_ref(g_object_finalized)
+
+    def is_finalizer_ref(ref):
+        try:
+            return isinstance(ref[2][0], str)
+        except Exception:
+            return False
+
+    def check_finalized():
+        gc.collect()
+        gc.collect()
+
+        if g_objects:
+            logger.warning('GObject not finalized: %s', name)
+
+        tup = finalizer.peek()
+        if tup is None:
+            return
+
+        logger.warning('%s not finalized', name)
+        logger.warning('References:')
+        for ref in gc.get_referrers(tup[0]):
+            if is_finalizer_ref(ref):
+                continue
+            if isinstance(ref, dict):
+                logger.warning('\n%s', pprint.pformat(ref))
+            else:
+                logger.warning(ref)
+
+    GLib.timeout_add_seconds(2, check_finalized)
