@@ -12,32 +12,49 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Optional
+
 import logging
 
 import nbxmpp
 from nbxmpp.client import Client as NBXMPPClient
+from nbxmpp.protocol import JID
 from nbxmpp.const import StreamError
 from nbxmpp.const import ConnectionType
 
 from gi.repository import GLib
-
-from gajim.common import passwords
-from gajim.common.nec import NetworkEvent
+from gi.repository import Gio
+from gi.repository import GObject
+from gi.repository import Gtk
 
 from gajim.common import app
 from gajim.common import helpers
 from gajim.common import modules
+from gajim.common import passwords
+from gajim.common.events import AccountConnected
+from gajim.common.events import MessageSent
+from gajim.common.events import AccountDisonnected
+from gajim.common.events import PasswordRequired
+from gajim.common.events import PlainConnection
+from gajim.common.events import SignedIn
+from gajim.common.events import Notification
+from gajim.common.events import StanzaReceived
+from gajim.common.events import StanzaSent
 from gajim.common.const import ClientState
+from gajim.common.const import SimpleClientState
 from gajim.common.helpers import get_custom_host
-from gajim.common.helpers import get_user_proxy
+from gajim.common.helpers import get_account_proxy
 from gajim.common.helpers import warn_about_plain_connection
 from gajim.common.helpers import get_resource
 from gajim.common.helpers import get_idle_status_message
+from gajim.common.helpers import Observable
 from gajim.common.idle import Monitor
+from gajim.common.idle import IdleMonitorManager
 from gajim.common.i18n import _
-
-from gajim.common.connection_handlers import ConnectionHandlers
-from gajim.common.connection_handlers_events import MessageSentEvent
+from gajim.common.structs import OutgoingMessage
 
 from gajim.gui.util import open_window
 
@@ -45,8 +62,12 @@ from gajim.gui.util import open_window
 log = logging.getLogger('gajim.client')
 
 
-class Client(ConnectionHandlers):
-    def __init__(self, account):
+IgnoredTlsErrorsT = Optional[set[Gio.TlsCertificateFlags]]
+
+
+class Client(Observable):
+    def __init__(self, account: str) -> None:
+        Observable.__init__(self, log)
         self._client = None
         self._account = account
         self.name = account
@@ -59,8 +80,6 @@ class Client(ConnectionHandlers):
         self._connect_machine_calls = 0
         self.addressing_supported = False
 
-        self.is_zeroconf = False
-        self.pep = {}
         self.roster_supported = True
 
         self._state = ClientState.DISCONNECTED
@@ -90,32 +109,30 @@ class Client(ConnectionHandlers):
             self._screensaver_handler_id = app.app.connect(
                 'notify::screensaver-active', self._screensaver_state_changed)
 
-        ConnectionHandlers.__init__(self)
-
-    def _set_state(self, state):
+    def _set_state(self, state: ClientState) -> None:
         log.info('State: %s', state)
         self._state = state
 
     @property
-    def state(self):
+    def state(self) -> ClientState:
         return self._state
 
     @property
-    def account(self):
+    def account(self) -> str:
         return self._account
 
     @property
-    def status(self):
+    def status(self) -> str:
         return self._status
 
     @property
-    def status_message(self):
+    def status_message(self) -> str:
         if self._idle_status_active():
             return self._idle_status_message
         return self._status_message
 
     @property
-    def priority(self):
+    def priority(self) -> int:
         return self._priority
 
     @property
@@ -127,19 +144,19 @@ class Client(ConnectionHandlers):
         return self._client.features
 
     @property
-    def local_address(self):
+    def local_address(self) -> Optional[str]:
         address = self._client.local_address
         if address is not None:
             return address.to_string().split(':')[0]
         return None
 
-    def set_remove_account(self, value):
+    def set_remove_account(self, value: bool) -> None:
         # Used by the RemoveAccount Assistant to make the Client
         # not react to any stream errors that happen while the
         # account is removed by the server and the connection is killed
         self._remove_account = value
 
-    def _create_client(self):
+    def _create_client(self) -> None:
         if self._destroyed:
             # If we disable an account cleanup() is called and all
             # modules are unregistered. Because disable_account() does not wait
@@ -179,34 +196,35 @@ class Client(ConnectionHandlers):
         for handler in modules.get_handlers(self):
             self._client.register_handler(handler)
 
-    def _on_resume_failed(self, _client, _signal_name):
+    def _on_resume_failed(self,
+                          _client: NBXMPPClient,
+                          _signal_name: str) -> None:
+
         log.info('Resume failed')
-        app.nec.push_incoming_event(NetworkEvent(
-            'our-show', account=self._account, show='offline'))
-        self.get_module('Chatstate').enabled = False
+        self.notify('resume-failed')
+
+    def _on_resume_successful(self,
+                              _client: NBXMPPClient,
+                              _signal_name: str) -> None:
 
-    def _on_resume_successful(self, _client, _signal_name):
         self._set_state(ClientState.CONNECTED)
         self._set_client_available()
 
         if self._status_sync_on_resume:
             self._status_sync_on_resume = False
             self.update_presence()
-        else:
-            # Normally show is updated when we receive a presence reflection.
-            # On resume, if show has not changed while offline, we don’t send
-            # a new presence so we have to trigger the event here.
-            app.nec.push_incoming_event(
-                NetworkEvent('our-show',
-                             account=self._account,
-                             show=self._status))
 
-    def _set_client_available(self):
+        self.notify('state-changed', SimpleClientState.CONNECTED)
+
+    def _set_client_available(self) -> None:
         self._set_state(ClientState.AVAILABLE)
-        app.nec.push_incoming_event(NetworkEvent('account-connected',
-                                                 account=self._account))
+        app.ged.raise_event(AccountConnected(account=self._account))
+
+    def disconnect(self,
+                   gracefully: bool,
+                   reconnect: bool,
+                   destroy_client: bool = False) -> None:
 
-    def disconnect(self, gracefully, reconnect, destroy_client=False):
         if self._state.is_disconnecting:
             log.warning('Disconnect already in progress')
             return
@@ -218,7 +236,10 @@ class Client(ConnectionHandlers):
         log.info('Starting to disconnect %s', self._account)
         self._client.disconnect(immediate=not gracefully)
 
-    def _on_disconnected(self, _client, _signal_name):
+    def _on_disconnected(self,
+                         _client: NBXMPPClient,
+                         _signal_name: str) -> None:
+
         log.info('Disconnect %s', self._account)
         self._set_state(ClientState.DISCONNECTED)
 
@@ -246,46 +267,47 @@ class Client(ConnectionHandlers):
                 app.settings.set_account_setting(self._account,
                                                  'resource',
                                                  'gajim.$rand')
+            if error == 'system-shutdown':
+                app.ged.raise_event(
+                    Notification(account=self._account,
+                                 type='server-shutdown',
+                                 title=_('Server Shutdown'),
+                                 text=text or _('The server was shut down.')))
 
         elif domain == StreamError.SASL:
             self._reconnect = False
             self._destroy_client = True
 
             if error in ('not-authorized', 'no-password'):
-                def _on_password(password):
+                def _on_password(password: str) -> None:
                     self.password = password
                     self._client.set_password(password)
                     self._prepare_for_connect()
 
-                app.nec.push_incoming_event(NetworkEvent(
-                    'password-required', conn=self, on_password=_on_password))
+                app.ged.raise_event(PasswordRequired(client=self,
+                                                     on_password=_on_password))
 
-            app.nec.push_incoming_event(
-                NetworkEvent('simple-notification',
-                             account=self._account,
-                             type_='connection-failed',
+            app.ged.raise_event(
+                Notification(account=self._account,
+                             type='connection-failed',
                              title=_('Authentication failed'),
                              text=text or error))
 
         if self._reconnect:
             self._after_disconnect()
             self._schedule_reconnect()
-            app.nec.push_incoming_event(
-                NetworkEvent('our-show', account=self._account, show='error'))
+            if not self._client.resumeable:
+                self.notify('state-changed', SimpleClientState.DISCONNECTED)
+            self.notify('state-changed', SimpleClientState.RESUME_IN_PROGRESS)
 
         else:
-            self.get_module('Chatstate').enabled = False
-            app.nec.push_incoming_event(NetworkEvent(
-                'our-show', account=self._account, show='offline'))
             self._after_disconnect()
+            self.notify('state-changed', SimpleClientState.DISCONNECTED)
 
-    def _after_disconnect(self):
+    def _after_disconnect(self) -> None:
         self._disable_reconnect_timer()
 
-        self.get_module('VCardAvatars').avatar_advertised = False
-
         app.proxy65_manager.disconnect(self._client)
-        self.terminate_sessions()
         self.get_module('Bytestream').remove_all_transfers()
 
         if self._destroy_client:
@@ -294,31 +316,39 @@ class Client(ConnectionHandlers):
             self._destroy_client = False
             self._create_client()
 
-        app.nec.push_incoming_event(NetworkEvent('account-disconnected',
-                                                 account=self._account))
+        app.ged.raise_event(AccountDisonnected(account=self._account))
 
-    def _on_connection_failed(self, _client, _signal_name):
+    def _on_connection_failed(self,
+                              _client: NBXMPPClient,
+                              _signal_name: str) -> None:
         self._schedule_reconnect()
 
-    def _on_connected(self, _client, _signal_name):
+    def _on_connected(self,
+                      _client: NBXMPPClient,
+                      _signal_name: str) -> None:
+
         self._set_state(ClientState.CONNECTED)
-        self.get_module('MUC').get_manager().reset_state()
         self.get_module('Discovery').discover_server_info()
         self.get_module('Discovery').discover_account_info()
         self.get_module('Discovery').discover_server_items()
-        self.get_module('Chatstate').enabled = True
-        self.get_module('MAM').reset_state()
 
-    def _on_stanza_sent(self, _client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-sent',
-                                                 account=self._account,
-                                                 stanza=stanza))
+    def _on_stanza_sent(self,
+                        _client: NBXMPPClient,
+                        _signal_name: str,
+                        stanza: Any) -> None:
 
-    def _on_stanza_received(self, _client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-received',
-                                                 account=self._account,
-                                                 stanza=stanza))
-    def get_own_jid(self):
+        app.ged.raise_event(StanzaSent(account=self._account,
+                                       stanza=stanza))
+
+    def _on_stanza_received(self,
+                            _client: NBXMPPClient,
+                            _signal_name: str,
+                            stanza: Any) -> None:
+
+        app.ged.raise_event(StanzaReceived(account=self._account,
+                                           stanza=stanza))
+
+    def get_own_jid(self) -> JID:
         """
         Return the last full JID we received on a bind event.
         In case we were never connected it returns the bare JID from config.
@@ -331,7 +361,7 @@ class Client(ConnectionHandlers):
         # This returns the bare jid
         return nbxmpp.JID.from_string(app.get_jid_from_account(self._account))
 
-    def change_status(self, show, message):
+    def change_status(self, show: str, message: str) -> None:
         if not message:
             message = ''
 
@@ -370,10 +400,8 @@ class Client(ConnectionHandlers):
 
         # We are connected
         if show == 'offline':
-            self.set_user_activity(None)
-            self.set_user_mood(None)
-            self.set_user_tune(None)
-            self.set_user_location(None)
+            self.get_module('UserTune').set_tune(None)
+            self.get_module('UserLocation').set_location(None)
             presence = self.get_module('Presence').get_presence(
                 typ='unavailable',
                 status=message,
@@ -387,7 +415,7 @@ class Client(ConnectionHandlers):
 
         self.update_presence()
 
-    def update_presence(self, include_muc=True):
+    def update_presence(self, include_muc: bool = True) -> None:
         status, message, idle = self.get_presence_state()
         self._priority = app.get_priority(self._account, status)
         self.get_module('Presence').send_presence(
@@ -399,34 +427,20 @@ class Client(ConnectionHandlers):
         if include_muc:
             self.get_module('MUC').update_presence()
 
-    def set_user_activity(self, activity):
-        self.get_module('UserActivity').set_activity(activity)
-
-    def set_user_mood(self, mood):
-        self.get_module('UserMood').set_mood(mood)
-
-    def set_user_tune(self, tune):
-        self.get_module('UserTune').set_tune(tune)
-
-    def set_user_location(self, location):
-        self.get_module('UserLocation').set_location(location)
-
-    def get_module(self, name):
+    def get_module(self, name: str):
         return modules.get(self._account, name)
 
     @helpers.call_counter
-    def connect_machine(self):
+    def connect_machine(self) -> None:
         log.info('Connect machine state: %s', self._connect_machine_calls)
         if self._connect_machine_calls == 1:
-            self.get_module('MetaContacts').get_metacontacts()
-        elif self._connect_machine_calls == 2:
             self.get_module('Delimiter').get_roster_delimiter()
-        elif self._connect_machine_calls == 3:
+        elif self._connect_machine_calls == 2:
             self.get_module('Roster').request_roster()
-        elif self._connect_machine_calls == 4:
+        elif self._connect_machine_calls == 3:
             self._finish_connect()
 
-    def _finish_connect(self):
+    def _finish_connect(self) -> None:
         self._status_sync_on_resume = False
         self._set_client_available()
 
@@ -435,21 +449,26 @@ class Client(ConnectionHandlers):
 
         self.get_module('Bookmarks').request_bookmarks()
         self.get_module('SoftwareVersion').set_enabled(True)
+        self.get_module('EntityTime').set_enabled(True)
+        self.get_module('LastActivity').set_enabled(True)
         self.get_module('Annotations').request_annotations()
         self.get_module('Blocking').get_blocking_list()
 
-        # Inform GUI we just signed in
-        app.nec.push_incoming_event(NetworkEvent(
-            'signed-in', account=self._account, conn=self))
+        if app.settings.get_account_setting(self._account, 'publish_tune'):
+            self.get_module('UserTune').set_enabled(True)
+
+        self.notify('state-changed', SimpleClientState.CONNECTED)
+
+        app.ged.raise_event(SignedIn(account=self._account, conn=self))
         modules.send_stored_publish(self._account)
 
-    def send_stanza(self, stanza):
+    def send_stanza(self, stanza: Any) -> None:
         """
         Send a stanza untouched
         """
         return self._client.send_stanza(stanza)
 
-    def send_message(self, message):
+    def send_message(self, message: OutgoingMessage) -> None:
         if not self._state.is_available:
             log.warning('Trying to send message while offline')
             return
@@ -457,11 +476,6 @@ class Client(ConnectionHandlers):
         stanza = self.get_module('Message').build_message_stanza(message)
         message.stanza = stanza
 
-        if message.contact is None:
-            # Only Single Message should have no contact
-            self._send_message(message)
-            return
-
         method = message.contact.settings.get('encryption')
         if not method:
             self._send_message(message)
@@ -476,31 +490,28 @@ class Client(ConnectionHandlers):
                                            message,
                                            self._send_message)
 
-    def _send_message(self, message):
+    def _send_message(self, message: OutgoingMessage) -> None:
         message.set_sent_timestamp()
         message.message_id = self.send_stanza(message.stanza)
 
-        app.nec.push_incoming_event(
-            MessageSentEvent(None, jid=message.jid, **vars(message)))
+        app.ged.raise_event(
+            MessageSent(jid=message.jid,
+                        account=message.account,
+                        message=message.message,
+                        chatstate=message.chatstate,
+                        timestamp=message.timestamp,
+                        additional_data=message.additional_data,
+                        label=message.label,
+                        correct_id=message.correct_id,
+                        message_id=message.message_id,
+                        play_sound=message.play_sound))
 
         if message.is_groupchat:
             return
 
         self.get_module('Message').log_message(message)
 
-    def send_messages(self, jids, message):
-        if not self._state.is_available:
-            log.warning('Trying to send message while offline')
-            return
-
-        for jid in jids:
-            message = message.copy()
-            message.contact = app.contacts.create_contact(jid, message.account)
-            stanza = self.get_module('Message').build_message_stanza(message)
-            message.stanza = stanza
-            self._send_message(message)
-
-    def _prepare_for_connect(self):
+    def _prepare_for_connect(self) -> None:
         custom_host = get_custom_host(self._account)
         if custom_host is not None:
             self._client.set_custom_host(*custom_host)
@@ -519,13 +530,13 @@ class Client(ConnectionHandlers):
                                             'use_plain_connection'):
             self._client.set_connection_types([ConnectionType.PLAIN])
 
-        proxy = get_user_proxy(self._account)
+        proxy = get_account_proxy(self._account)
         if proxy is not None:
             self._client.set_proxy(proxy)
 
         self.connect()
 
-    def connect(self, ignored_tls_errors=None):
+    def connect(self, ignored_tls_errors: IgnoredTlsErrorsT = None) -> None:
         if self._state not in (ClientState.DISCONNECTED,
                                ClientState.RECONNECT_SCHEDULED):
             # Do not try to reco while we are already trying
@@ -537,29 +548,27 @@ class Client(ConnectionHandlers):
         self._reconnect = True
         self._disable_reconnect_timer()
         self._set_state(ClientState.CONNECTING)
+        self.notify('state-changed', SimpleClientState.CONNECTING)
 
         if warn_about_plain_connection(self._account,
                                        self._client.connection_types):
-            app.nec.push_incoming_event(NetworkEvent(
-                'plain-connection',
-                account=self._account,
-                connect=self._client.connect,
-                abort=self._abort_reconnect))
+            app.ged.raise_event(PlainConnection(account=self._account,
+                                                connect=self._client.connect,
+                                                abort=self._abort_reconnect))
             return
 
         self._client.connect()
 
-    def _schedule_reconnect(self):
+    def _schedule_reconnect(self) -> None:
         self._set_state(ClientState.RECONNECT_SCHEDULED)
         log.info("Reconnect to %s in 3s", self._account)
         self._reconnect_timer_source = GLib.timeout_add_seconds(
             3, self._prepare_for_connect)
 
-    def _abort_reconnect(self):
+    def _abort_reconnect(self) -> None:
         self._set_state(ClientState.DISCONNECTED)
         self._disable_reconnect_timer()
-        app.nec.push_incoming_event(
-            NetworkEvent('our-show', account=self._account, show='offline'))
+        self.notify('state-changed', SimpleClientState.DISCONNECTED)
 
         if self._destroy_client:
             self._client.destroy()
@@ -567,12 +576,12 @@ class Client(ConnectionHandlers):
             self._destroy_client = False
             self._create_client()
 
-    def _disable_reconnect_timer(self):
+    def _disable_reconnect_timer(self) -> None:
         if self._reconnect_timer_source is not None:
             GLib.source_remove(self._reconnect_timer_source)
             self._reconnect_timer_source = None
 
-    def _idle_state_changed(self, monitor):
+    def _idle_state_changed(self, monitor: IdleMonitorManager) -> None:
         state = monitor.state.value
 
         if monitor.is_awake():
@@ -592,7 +601,7 @@ class Client(ConnectionHandlers):
                 state, self._status_message)
             self._update_status()
 
-    def _update_status(self):
+    def _update_status(self) -> None:
         if not self._idle_status_enabled:
             return
 
@@ -602,7 +611,7 @@ class Client(ConnectionHandlers):
         else:
             self._status_sync_on_resume = True
 
-    def _idle_status_active(self):
+    def _idle_status_active(self) -> bool:
         if not Monitor.is_available():
             return False
 
@@ -611,17 +620,19 @@ class Client(ConnectionHandlers):
 
         return self._idle_status != 'online'
 
-    def get_presence_state(self):
+    def get_presence_state(self) -> tuple[str, str, bool]:
         if self._idle_status_active():
             return self._idle_status, self._idle_status_message, True
         return self._status, self._status_message, False
 
     @staticmethod
-    def _screensaver_state_changed(application, _param):
+    def _screensaver_state_changed(application: Gtk.Application,
+                                   _param: GObject.ParamSpec) -> None:
         active = application.get_property('screensaver-active')
         Monitor.set_extended_away(active)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
+        self.disconnect_signals()
         self._destroyed = True
         if Monitor.is_available():
             Monitor.disconnect(self._idle_handler_id)
@@ -635,7 +646,7 @@ class Client(ConnectionHandlers):
                 self._client.unregister_handler(handler)
         modules.unregister_modules(self)
 
-    def quit(self, kill_core):
+    def quit(self, kill_core: bool) -> None:
         if kill_core and self._state in (ClientState.CONNECTING,
                                          ClientState.CONNECTED,
                                          ClientState.AVAILABLE):
