@@ -21,6 +21,7 @@ from typing import Dict  # pylint: disable=unused-import
 
 import logging
 import socket
+import uuid
 from enum import IntEnum, unique
 
 import nbxmpp
@@ -153,16 +154,13 @@ class JingleTransportSocks5(JingleTransport):
     def parse_transport_stanza(self, transport):
         candidates = []
         for candidate in transport.iterTags('candidate'):
-            typ = 'direct' # default value
-            if candidate.has_attr('type'):
-                typ = candidate['type']
             cand = {
                 'state': 0,
                 'target': self.ourjid,
                 'host': candidate['host'],
-                'port': int(candidate['port']),
+                'port': int(candidate.get('port', 1080)),
                 'candidate_id': candidate['cid'],
-                'type': typ,
+                'type': candidate.get('type', 'direct'),
                 'priority': candidate['priority']
             }
             candidates.append(cand)
@@ -436,7 +434,7 @@ class JingleTransportICEUDP(JingleTransport):
             'network': '0',
             'port': candidate.port,
             'priority': int(candidate.priority), # hack
-            'id': app.get_an_id()
+            'id': str(uuid.uuid4())
         }
         if candidate.type in types:
             attrs['type'] = types[candidate.type]
