@@ -12,7 +12,11 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import sys
+from __future__ import annotations
+
+from typing import Any
+from typing import cast
+
 import locale
 import logging
 from collections import defaultdict
@@ -23,7 +27,9 @@ from gi.repository import Pango
 from gi.repository import GObject
 
 from gajim.common import app
+from gajim.common import ged
 from gajim.common import passwords
+from gajim.common.events import AccountDisonnected
 from gajim.common.i18n import _
 from gajim.common.i18n import Q_
 
@@ -32,8 +38,10 @@ from .dialogs import ConfirmationDialog
 from .const import Setting
 from .const import SettingKind
 from .const import SettingType
+from .controls.base import BaseControl
 from .settings import SettingsDialog
 from .settings import SettingsBox
+from .settings import PopoverSetting
 from .util import open_window
 
 
@@ -41,7 +49,7 @@ log = logging.getLogger('gajim.gui.accounts')
 
 
 class AccountsWindow(Gtk.ApplicationWindow):
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
         self.set_position(Gtk.WindowPosition.CENTER)
@@ -50,8 +58,8 @@ class AccountsWindow(Gtk.ApplicationWindow):
         self.set_default_size(700, 550)
         self.set_resizable(True)
         self.set_title(_('Accounts'))
-        self._need_relogin = {}
-        self._accounts = {}
+        self._need_relogin: dict[str, list[str]] = {}
+        self._accounts: dict[str, Account] = {}
 
         self._menu = AccountMenu()
         self._settings = Settings()
@@ -62,9 +70,6 @@ class AccountsWindow(Gtk.ApplicationWindow):
         self.add(box)
 
         for account in app.get_accounts_sorted():
-            if account == 'Local':
-                # Disable zeroconf support until its working again
-                continue
             self.add_account(account, initial=True)
 
         self._menu.connect('menu-activated', self._on_menu_activated)
@@ -73,7 +78,10 @@ class AccountsWindow(Gtk.ApplicationWindow):
 
         self.show_all()
 
-    def _on_menu_activated(self, _listbox, account, name):
+    def _on_menu_activated(self,
+                           _listbox: Gtk.ListBox,
+                           account: str,
+                           name: str) -> None:
         if name == 'back':
             self._settings.set_page('add-account')
             self._check_relogin()
@@ -82,38 +90,37 @@ class AccountsWindow(Gtk.ApplicationWindow):
         else:
             self._settings.set_page(name)
 
-    def _on_key_press(self, _widget, event):
+    def _on_key_press(self,
+                      _widget: AccountsWindow,
+                      event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def _on_destroy(self, *args):
+    def _on_destroy(self, _widget: AccountsWindow) -> None:
         self._check_relogin()
+        app.check_finalize(self)
 
-    def update_account_label(self, account):
+    def update_account_label(self, account: str) -> None:
         self._accounts[account].update_account_label()
 
-    def update_proxy_list(self):
+    def update_proxy_list(self) -> None:
         for account in self._accounts:
             self._settings.update_proxy_list(account)
 
-    def _check_relogin(self):
-        for account in self._need_relogin:
+    def _check_relogin(self) -> None:
+        for account, r_settings in self._need_relogin.items():
             settings = self._get_relogin_settings(account)
             active = app.settings.get_account_setting(account, 'active')
-            if settings != self._need_relogin[account]:
+            if settings != r_settings:
                 self._need_relogin[account] = settings
                 if active:
                     self._relog(account)
                 break
 
-    def _relog(self, account):
+    def _relog(self, account: str) -> None:
         if not app.account_is_connected(account):
             return
 
-        if account == app.ZEROCONF_ACC_NAME:
-            app.connections[app.ZEROCONF_ACC_NAME].update_details()
-            return
-
         def relog():
             app.connections[account].disconnect(gracefully=True,
                                                 reconnect=True,
@@ -131,49 +138,43 @@ class AccountsWindow(Gtk.ApplicationWindow):
             transient_for=self).show()
 
     @staticmethod
-    def _get_relogin_settings(account):
-        if account == app.ZEROCONF_ACC_NAME:
-            settings = ['zeroconf_first_name', 'zeroconf_last_name',
-                        'zeroconf_jabber_id', 'zeroconf_email']
-        else:
-            settings = ['client_cert', 'proxy', 'resource',
-                        'use_custom_host', 'custom_host', 'custom_port']
+    def _get_relogin_settings(account: str) -> list[str]:
+        settings = [
+            'client_cert',
+            'proxy',
+            'resource',
+            'use_custom_host',
+            'custom_host',
+            'custom_port'
+        ]
 
-        values = []
+        values: list[Any] = []
         for setting in settings:
             values.append(app.settings.get_account_setting(account, setting))
         return values
 
     @staticmethod
-    def on_remove_account(account):
-        if app.events.get_events(account):
-            app.interface.raise_dialog('unread-events-on-remove-account')
-            return
-
-        if app.settings.get_account_setting(account, 'is_zeroconf'):
-            # Should never happen as button is insensitive
-            return
-
+    def on_remove_account(account: str) -> None:
         open_window('RemoveAccount', account=account)
 
-    def remove_account(self, account):
+    def remove_account(self, account: str) -> None:
         del self._need_relogin[account]
         self._accounts[account].remove()
 
-    def add_account(self, account, initial=False):
+    def add_account(self, account: str, initial: bool = False) -> None:
         self._need_relogin[account] = self._get_relogin_settings(account)
         self._accounts[account] = Account(account, self._menu, self._settings)
         if not initial:
             self._accounts[account].show()
 
-    def select_account(self, account):
+    def select_account(self, account: str) -> None:
         try:
             self._accounts[account].select()
         except KeyError:
             log.warning('select_account() failed, account %s not found',
                         account)
 
-    def enable_account(self, account, state):
+    def enable_account(self, account: str, state: bool) -> None:
         self._accounts[account].enable_account(state)
 
 
@@ -189,18 +190,18 @@ class Settings(Gtk.ScrolledWindow):
         self.get_style_context().add_class('accounts-settings')
 
         self.add(self._stack)
-        self._page_signal_ids = {}
-        self._pages = defaultdict(list)
+        self._page_signal_ids: dict[GenericSettingPage, int] = {}
+        self._pages: dict[str, list[GenericSettingPage]] = defaultdict(list)
 
-    def add_page(self, page):
+    def add_page(self, page: GenericSettingPage) -> None:
         self._pages[page.account].append(page)
-        self._stack.add_named(page, '%s-%s' % (page.account, page.name))
+        self._stack.add_named(page, f'{page.account}-{page.name}')
         self._page_signal_ids[page] = page.connect_signal(self._stack)
 
-    def set_page(self, name):
+    def set_page(self, name: str) -> None:
         self._stack.set_visible_child_name(name)
 
-    def remove_account(self, account):
+    def remove_account(self, account: str) -> None:
         for page in self._pages[account]:
             signal_id = self._page_signal_ids[page]
             del self._page_signal_ids[page]
@@ -209,10 +210,11 @@ class Settings(Gtk.ScrolledWindow):
             page.destroy()
         del self._pages[account]
 
-    def update_proxy_list(self, account):
+    def update_proxy_list(self, account: str) -> None:
         for page in self._pages[account]:
             if page.name != 'connection':
                 continue
+            assert isinstance(page, ConnectionPage)
             page.update_proxy_entries()
 
 
@@ -222,7 +224,7 @@ class AccountMenu(Gtk.Box):
         'menu-activated': (GObject.SignalFlags.RUN_FIRST, None, (str, str)),
     }
 
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.Box.__init__(self)
         self.set_hexpand(False)
         self.set_size_request(160, -1)
@@ -245,33 +247,40 @@ class AccountMenu(Gtk.Box):
         self.add(self._stack)
 
     @staticmethod
-    def _sort_func(row1, row2):
-        if row1.label == 'Local':
-            return -1
+    def _sort_func(row1: AccountRow, row2: AccountRow) -> int:
         return locale.strcoll(row1.label.lower(), row2.label.lower())
 
-    def add_account(self, row):
+    def add_account(self, row: AccountRow) -> None:
         self._accounts_listbox.add(row)
         sub_menu = AccountSubMenu(row.account)
-        self._stack.add_named(sub_menu, '%s-menu' % row.account)
+        self._stack.add_named(sub_menu, f'{row.account}-menu')
 
         sub_menu.connect('row-activated', self._on_sub_menu_row_activated)
 
-    def remove_account(self, row):
+    def remove_account(self, row: AccountRow) -> None:
         if self._stack.get_visible_child_name() != 'accounts':
             # activate 'back' button
-            self._stack.get_visible_child().get_row_at_index(1).emit('activate')
+            listbox = cast(Gtk.ListBox, self._stack.get_visible_child())
+            back_row = cast(Gtk.ListBoxRow, listbox.get_row_at_index(1))
+            back_row.emit('activate')
         self._accounts_listbox.remove(row)
         sub_menu = self._stack.get_child_by_name('%s-menu' % row.account)
         self._stack.remove(sub_menu)
         row.destroy()
         sub_menu.destroy()
 
-    def _on_account_row_activated(self, _listbox, row):
+    def _on_account_row_activated(self,
+                                  _listbox: Gtk.ListBox,
+                                  row: AccountRow
+                                  ) -> None:
         self._stack.set_visible_child_name('%s-menu' % row.account)
-        self._stack.get_visible_child().get_row_at_index(2).emit('activate')
+        listbox = cast(Gtk.ListBox, self._stack.get_visible_child())
+        listbox_row = cast(Gtk.ListBoxRow, listbox.get_row_at_index(2))
+        listbox_row.emit('activate')
 
-    def _on_sub_menu_row_activated(self, listbox, row):
+    def _on_sub_menu_row_activated(self,
+                                   listbox: AccountSubMenu,
+                                   row: MenuItem) -> None:
         if row.name == 'back':
             self._stack.set_visible_child_full(
                 'accounts', Gtk.StackTransitionType.OVER_RIGHT)
@@ -282,11 +291,12 @@ class AccountMenu(Gtk.Box):
         else:
             self.emit('menu-activated',
                       listbox.account,
-                      '%s-%s' % (listbox.account, row.name))
+                      f'{listbox.account}-{row.name}')
 
-    def update_account_label(self, account):
+    def update_account_label(self, account: str) -> None:
         self._accounts_listbox.invalidate_sort()
-        sub_menu = self._stack.get_child_by_name('%s-menu' % account)
+        sub_menu = cast(
+            AccountSubMenu, self._stack.get_child_by_name(f'{account}-menu'))
         sub_menu.update()
 
 
@@ -296,7 +306,7 @@ class AccountSubMenu(Gtk.ListBox):
         'update': (GObject.SignalFlags.RUN_FIRST, None, (str,))
     }
 
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
         Gtk.ListBox.__init__(self)
         self.set_vexpand(True)
         self.set_hexpand(True)
@@ -307,22 +317,21 @@ class AccountSubMenu(Gtk.ListBox):
         self.add(AccountLabelMenuItem(self, self._account))
         self.add(BackMenuItem())
         self.add(PageMenuItem('general', _('General')))
-        if account != 'Local':
-            self.add(PageMenuItem('privacy', _('Privacy')))
-            self.add(PageMenuItem('connection', _('Connection')))
-            self.add(PageMenuItem('advanced', _('Advanced')))
-            self.add(RemoveMenuItem())
+        self.add(PageMenuItem('privacy', _('Privacy')))
+        self.add(PageMenuItem('connection', _('Connection')))
+        self.add(PageMenuItem('advanced', _('Advanced')))
+        self.add(RemoveMenuItem())
 
     @property
-    def account(self):
+    def account(self) -> str:
         return self._account
 
-    def update(self):
+    def update(self) -> None:
         self.emit('update', self._account)
 
 
 class MenuItem(Gtk.ListBoxRow):
-    def __init__(self, name):
+    def __init__(self, name: str) -> None:
         Gtk.ListBoxRow.__init__(self)
         self._name = name
         self._box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL,
@@ -332,12 +341,12 @@ class MenuItem(Gtk.ListBoxRow):
         self.add(self._box)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
 
 class RemoveMenuItem(MenuItem):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__('remove')
         self._label.set_text(_('Remove'))
         image = Gtk.Image.new_from_icon_name('user-trash-symbolic',
@@ -351,7 +360,7 @@ class RemoveMenuItem(MenuItem):
 
 
 class AccountLabelMenuItem(MenuItem):
-    def __init__(self, parent, account):
+    def __init__(self, parent: AccountSubMenu, account: str) -> None:
         super().__init__('account-label')
         self._update_account_label(parent, account)
 
@@ -372,13 +381,16 @@ class AccountLabelMenuItem(MenuItem):
 
         parent.connect('update', self._update_account_label)
 
-    def _update_account_label(self, _listbox, account):
+    def _update_account_label(self,
+                              _listbox: Gtk.ListBox,
+                              account: str
+                              ) -> None:
         account_label = app.get_account_label(account)
         self._label.set_text(account_label)
 
 
 class BackMenuItem(MenuItem):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__('back')
         self.set_selectable(False)
 
@@ -393,7 +405,7 @@ class BackMenuItem(MenuItem):
 
 
 class PageMenuItem(MenuItem):
-    def __init__(self, name, label):
+    def __init__(self, name: str, label: str) -> None:
         super().__init__(name)
 
         if name == 'general':
@@ -415,56 +427,57 @@ class PageMenuItem(MenuItem):
 
 
 class Account:
-    def __init__(self, account, menu, settings):
+    def __init__(self,
+                 account: str,
+                 menu: AccountMenu,
+                 settings: Settings
+                 ) -> None:
         self._account = account
         self._menu = menu
         self._settings = settings
 
-        if account == app.ZEROCONF_ACC_NAME:
-            self._settings.add_page(ZeroConfPage(account))
-        else:
-            self._settings.add_page(GeneralPage(account))
-            self._settings.add_page(ConnectionPage(account))
-            self._settings.add_page(PrivacyPage(account))
-            self._settings.add_page(AdvancedPage(account))
+        self._settings.add_page(GeneralPage(account))
+        self._settings.add_page(ConnectionPage(account))
+        self._settings.add_page(PrivacyPage(account))
+        self._settings.add_page(AdvancedPage(account))
 
         self._account_row = AccountRow(account)
         self._menu.add_account(self._account_row)
 
-    def select(self):
+    def select(self) -> None:
         self._account_row.emit('activate')
 
-    def show(self):
+    def show(self) -> None:
         self._menu.show_all()
         self._settings.show_all()
         self.select()
 
-    def remove(self):
+    def remove(self) -> None:
         self._menu.remove_account(self._account_row)
         self._settings.remove_account(self._account)
 
-    def update_account_label(self):
+    def update_account_label(self) -> None:
         self._account_row.update_account_label()
         self._menu.update_account_label(self._account)
 
-    def enable_account(self, state):
+    def enable_account(self, state: bool) -> None:
         self._account_row.enable_account(state)
 
     @property
-    def menu(self):
+    def menu(self) -> AccountMenu:
         return self._menu
 
     @property
-    def account(self):
+    def account(self) -> str:
         return self._account
 
     @property
-    def settings(self):
+    def settings(self) -> str:
         return self._account
 
 
 class AccountRow(Gtk.ListBoxRow):
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
         Gtk.ListBoxRow.__init__(self)
         self.set_selectable(False)
 
@@ -494,17 +507,6 @@ class AccountRow(Gtk.ListBoxRow):
         self._switch_state_label.set_valign(Gtk.Align.CENTER)
         self._set_label(account_enabled)
 
-        if (self._account == app.ZEROCONF_ACC_NAME and
-                not app.is_installed('ZEROCONF')):
-            self._switch.set_active(False)
-            self.set_activatable(False)
-            self.set_sensitive(False)
-            if sys.platform in ('win32', 'darwin'):
-                tooltip = _('Please check if Bonjour is installed.')
-            else:
-                tooltip = _('Please check if Avahi is installed.')
-            self.set_tooltip_text(tooltip)
-
         self._switch.connect(
             'state-set', self._on_enable_switch, self._account)
 
@@ -516,28 +518,42 @@ class AccountRow(Gtk.ListBoxRow):
         self.add(box)
 
     @property
-    def account(self):
+    def account(self) -> str:
         return self._account
 
     @property
-    def label(self):
+    def label(self) -> str:
         return self._label.get_text()
 
-    def update_account_label(self):
+    def update_account_label(self) -> None:
         self._label.set_text(app.get_account_label(self._account))
 
-    def enable_account(self, state):
+    def enable_account(self, state: bool) -> None:
         self._switch.set_state(state)
         self._set_label(state)
 
-    def _set_label(self, active):
+    def _set_label(self, active: bool) -> None:
         text = Q_('?switch:On') if active else Q_('?switch:Off')
         self._switch_state_label.set_text(text)
 
-    def _on_enable_switch(self, switch, state, account):
-        def _disable():
-            app.connections[account].change_status('offline', 'offline')
+    def _on_enable_switch(self,
+                          switch: Gtk.Switch,
+                          state: bool,
+                          account: str
+                          ) -> int:
+        def _on_disconnect(event: AccountDisonnected) -> None:
+            if event.account != account:
+                return
+            app.ged.remove_event_handler('account-disconnected',
+                                         ged.CORE,
+                                         _on_disconnect)
             app.interface.disable_account(account)
+
+        def _disable() -> None:
+            app.ged.register_event_handler('account-disconnected',
+                                           ged.CORE,
+                                           _on_disconnect)
+            app.connections[account].change_status('offline', 'offline')
             switch.set_state(state)
             self._set_label(state)
 
@@ -548,6 +564,7 @@ class AccountRow(Gtk.ListBoxRow):
         if (account in app.connections and
                 not app.connections[account].state.is_disconnected):
             # Connecting or connected
+            window = cast(Gtk.Window, self.get_toplevel())
             ConfirmationDialog(
                 _('Disable Account'),
                 _('Account %s is still connected') % account,
@@ -557,7 +574,7 @@ class AccountRow(Gtk.ListBoxRow):
                  DialogButton.make('Remove',
                                    text=_('_Disable Account'),
                                    callback=_disable)],
-                transient_for=self.get_toplevel()).show()
+                transient_for=window).show()
             return Gdk.EVENT_STOP
 
         if state:
@@ -569,7 +586,7 @@ class AccountRow(Gtk.ListBoxRow):
 
 
 class AddNewAccountPage(Gtk.Box):
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.Box.__init__(self,
                          orientation=Gtk.Orientation.VERTICAL,
                          spacing=18)
@@ -581,7 +598,7 @@ class AddNewAccountPage(Gtk.Box):
             'org.gajim.Gajim-symbolic',
             100,
             self.get_scale_factor(),
-            0)
+            Gtk.IconLookupFlags.FORCE_SIZE)
         self.add(Gtk.Image.new_from_pixbuf(pixbuf))
 
         button = Gtk.Button(label=_('Add Account'))
@@ -592,7 +609,10 @@ class AddNewAccountPage(Gtk.Box):
 
 
 class GenericSettingPage(Gtk.Box):
-    def __init__(self, account, settings):
+
+    name = ''
+
+    def __init__(self, account: str, settings: list[Setting]) -> None:
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=12)
         self.set_valign(Gtk.Align.START)
         self.set_vexpand(True)
@@ -610,11 +630,11 @@ class GenericSettingPage(Gtk.Box):
 
         self.pack_end(self.listbox, True, True, 0)
 
-    def connect_signal(self, stack):
+    def connect_signal(self, stack: Gtk.Stack) -> int:
         return stack.connect('notify::visible-child',
                              self._on_visible_child_changed)
 
-    def _on_visible_child_changed(self, stack, _param):
+    def _on_visible_child_changed(self, stack: Gtk.Stack, _param: Any) -> None:
         if self == stack.get_visible_child():
             self.listbox.update_states()
 
@@ -623,7 +643,7 @@ class GeneralPage(GenericSettingPage):
 
     name = 'general'
 
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
 
         settings = [
             Setting(SettingKind.ENTRY, _('Label'),
@@ -634,7 +654,10 @@ class GeneralPage(GenericSettingPage):
                     SettingType.ACCOUNT_CONFIG, 'account_color',
                     desc=_('Recognize your account by color')),
 
-            Setting(SettingKind.LOGIN, _('Login'), SettingType.DIALOG,
+            Setting(SettingKind.LOGIN,
+                    _('Login'),
+                    SettingType.DIALOG,
+                    desc=_('Change your account’s password, etc.'),
                     bind='account::anonymous_auth',
                     inverted=True,
                     props={'dialog': LoginDialog}),
@@ -670,15 +693,16 @@ class GeneralPage(GenericSettingPage):
         ]
         GenericSettingPage.__init__(self, account, settings)
 
-    def _on_account_name_change(self, *args):
-        self.get_toplevel().update_account_label(self.account)
+    def _on_account_name_change(self, *args: Any) -> None:
+        window = cast(AccountsWindow, self.get_toplevel())
+        window.update_account_label(self.account)
 
 
 class PrivacyPage(GenericSettingPage):
 
     name = 'privacy'
 
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
         self._account = account
 
         history_max_age = {
@@ -700,17 +724,26 @@ class PrivacyPage(GenericSettingPage):
         settings = [
             Setting(SettingKind.SWITCH, _('Idle Time'),
                     SettingType.ACCOUNT_CONFIG, 'send_idle_time',
+                    callback=self._send_idle_time,
                     desc=_('Disclose the time of your last activity')),
 
             Setting(SettingKind.SWITCH, _('Local System Time'),
                     SettingType.ACCOUNT_CONFIG, 'send_time_info',
+                    callback=self._send_time_info,
                     desc=_('Disclose the local system time of the '
                            'device Gajim runs on')),
 
-            Setting(SettingKind.SWITCH, _('Client / Operating System'),
+            Setting(SettingKind.SWITCH, _('Operating System'),
                     SettingType.ACCOUNT_CONFIG, 'send_os_info',
-                    desc=_('Disclose information about the client '
-                           'and operating system you currently use')),
+                    callback=self._send_os_info,
+                    desc=_('Disclose information about the '
+                           'operating system you currently use')),
+
+            Setting(SettingKind.SWITCH, _('Media Playback'),
+                    SettingType.ACCOUNT_CONFIG, 'publish_tune',
+                    callback=self._publish_tune,
+                    desc=_('Disclose information about media that is '
+                           'currently being played on your system.')),
 
             Setting(SettingKind.SWITCH, _('Ignore Unknown Contacts'),
                     SettingType.ACCOUNT_CONFIG, 'ignore_unknown_contacts',
@@ -764,31 +797,51 @@ class PrivacyPage(GenericSettingPage):
                     'chat_history_max_age',
                     props={'entries': history_max_age},
                     desc=_('How long Gajim should keep your chat history')),
+
+            Setting(SettingKind.ACTION,
+                    _('Export Chat History'),
+                    SettingType.ACTION,
+                    '-export-history',
+                    props={'account': account},
+                    desc=_('Export your chat history from Gajim'))
         ]
         GenericSettingPage.__init__(self, account, settings)
 
     @staticmethod
-    def _reset_send_chatstate(button):
+    def _reset_send_chatstate(button: Gtk.Button) -> None:
         button.set_sensitive(False)
         app.settings.set_contact_settings('send_chatstate', None)
 
     @staticmethod
-    def _reset_gc_send_chatstate(button):
+    def _reset_gc_send_chatstate(button: Gtk.Button) -> None:
         button.set_sensitive(False)
         app.settings.set_group_chat_settings('send_chatstate', None)
 
-    def _send_read_marker(self, state, _data):
+    def _send_idle_time(self, state: bool, _data: Any) -> None:
+        app.connections[self._account].get_module('LastActivity').set_enabled(state)
+
+    def _send_time_info(self, state: bool, _data: Any) -> None:
+        app.connections[self._account].get_module('EntityTime').set_enabled(state)
+
+    def _send_os_info(self, state: bool, _data: Any) -> None:
+        app.connections[self._account].get_module('SoftwareVersion').set_enabled(state)
+
+    def _publish_tune(self, state: bool, _data: Any) -> None:
+        app.connections[self._account].get_module('UserTune').set_enabled(state)
+
+    def _send_read_marker(self, state: bool, _data: Any) -> None:
         app.settings.set_account_setting(
             self._account, 'send_marker_default', state)
         app.settings.set_account_setting(
             self._account, 'gc_send_marker_private_default', state)
 
-    def _reset_send_read_marker(self, button):
+    def _reset_send_read_marker(self, button: Gtk.Button) -> None:
         button.set_sensitive(False)
         app.settings.set_contact_settings('send_marker', None)
         app.settings.set_group_chat_settings(
             'send_marker', None, context='private')
-        for ctrl in app.interface.msg_win_mgr.get_controls(acct=self._account):
+        for ctrl in app.window.get_controls(account=self._account):
+            assert not isinstance(ctrl, BaseControl)
             ctrl.update_actions()
 
 
@@ -796,7 +849,7 @@ class ConnectionPage(GenericSettingPage):
 
     name = 'connection'
 
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
 
         settings = [
             Setting(SettingKind.POPOVER, _('Proxy'),
@@ -825,32 +878,32 @@ class ConnectionPage(GenericSettingPage):
                     'confirm_unencrypted_connection',
                     desc=_('Show a confirmation dialog before connecting '
                            'unencrypted')),
-            ]
+        ]
         GenericSettingPage.__init__(self, account, settings)
 
     @staticmethod
-    def _get_proxies():
+    def _get_proxies() -> dict[str, str]:
         return {proxy: proxy for proxy in app.settings.get_proxies()}
 
     @staticmethod
-    def _on_proxy_edit(*args):
+    def _on_proxy_edit(*args: Any) -> None:
         open_window('ManageProxies')
 
-    def update_proxy_entries(self):
-        self.listbox.get_setting('proxy').update_entries(self._get_proxies())
+    def update_proxy_entries(self) -> None:
+        popover_row = cast(PopoverSetting, self.listbox.get_setting('proxy'))
+        popover_row.update_entries(self._get_proxies())
 
 
 class AdvancedPage(GenericSettingPage):
 
     name = 'advanced'
 
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
 
         settings = [
             Setting(SettingKind.SWITCH, _('Contact Information'),
                     SettingType.ACCOUNT_CONFIG, 'request_user_data',
-                    desc=_('Request contact information '
-                           '(Mood, Activity, Tune, Location)')),
+                    desc=_('Request contact information (Tune, Location)')),
 
             Setting(SettingKind.SWITCH, _('Accept all Contact Requests'),
                     SettingType.ACCOUNT_CONFIG, 'autoauth',
@@ -861,62 +914,17 @@ class AdvancedPage(GenericSettingPage):
                     props={'entries': {'httpupload': _('Upload Files'),
                                        'jingle': _('Send Files Directly')}},
                     desc=_('Preferred file transfer mechanism for '
-                           'file drag&drop on a chat window'))
-            ]
+                           'file drag&drop on a chat window')),
+            Setting(SettingKind.SWITCH, _('Security Labels'),
+                    SettingType.ACCOUNT_CONFIG, 'enable_security_labels',
+                    desc=_('Show labels describing confidentiality of '
+                           'messages, if the server supports XEP-0258'))
+        ]
         GenericSettingPage.__init__(self, account, settings)
 
 
-class ZeroConfPage(GenericSettingPage):
-
-    name = 'general'
-
-    def __init__(self, account):
-
-        settings = [
-            Setting(SettingKind.DIALOG, _('Profile'),
-                    SettingType.DIALOG,
-                    props={'dialog': ZeroconfProfileDialog}),
-
-            Setting(SettingKind.SWITCH, _('Connect on startup'),
-                    SettingType.ACCOUNT_CONFIG, 'autoconnect',
-                    desc=_('Use environment variable')),
-
-            Setting(SettingKind.SWITCH,
-                    _('Save conversations for all contacts'),
-                    SettingType.ACCOUNT_CONFIG, 'no_log_for',
-                    desc=_('Store conversations on the harddrive')),
-
-            Setting(SettingKind.SWITCH, _('Global Status'),
-                    SettingType.ACCOUNT_CONFIG, 'sync_with_global_status',
-                    desc=_('Synchronize the status of all accounts')),
-            ]
-
-        GenericSettingPage.__init__(self, account, settings)
-
-
-class ZeroconfProfileDialog(SettingsDialog):
-    def __init__(self, account, parent):
-
-        settings = [
-            Setting(SettingKind.ENTRY, _('First Name'),
-                    SettingType.ACCOUNT_CONFIG, 'zeroconf_first_name'),
-
-            Setting(SettingKind.ENTRY, _('Last Name'),
-                    SettingType.ACCOUNT_CONFIG, 'zeroconf_last_name'),
-
-            Setting(SettingKind.ENTRY, _('XMPP Address'),
-                    SettingType.ACCOUNT_CONFIG, 'zeroconf_jabber_id'),
-
-            Setting(SettingKind.ENTRY, _('Email'),
-                    SettingType.ACCOUNT_CONFIG, 'zeroconf_email'),
-            ]
-
-        SettingsDialog.__init__(self, parent, _('Profile'),
-                                Gtk.DialogFlags.MODAL, settings, account)
-
-
 class PriorityDialog(SettingsDialog):
-    def __init__(self, account, parent):
+    def __init__(self, account: str, parent: Gtk.Window) -> None:
 
         neg_priority = app.settings.get('enable_negative_priority')
         if neg_priority:
@@ -935,14 +943,14 @@ class PriorityDialog(SettingsDialog):
                     bind='account::adjust_priority_with_status',
                     inverted=True,
                     props={'range_': range_}),
-            ]
+        ]
 
         SettingsDialog.__init__(self, parent, _('Priority'),
                                 Gtk.DialogFlags.MODAL, settings, account)
 
         self.connect('destroy', self.on_destroy)
 
-    def on_destroy(self, *args):
+    def on_destroy(self, *args: Any) -> None:
         # Update priority
         if self.account not in app.connections:
             return
@@ -952,7 +960,7 @@ class PriorityDialog(SettingsDialog):
 
 
 class CutstomHostnameDialog(SettingsDialog):
-    def __init__(self, account, parent):
+    def __init__(self, account: str, parent: Gtk.Window) -> None:
 
         type_values = ('START TLS', 'DIRECT TLS', 'PLAIN')
 
@@ -974,14 +982,14 @@ class CutstomHostnameDialog(SettingsDialog):
                     SettingType.ACCOUNT_CONFIG, 'custom_type',
                     bind='account::use_custom_host',
                     props={'combo_items': type_values}),
-            ]
+        ]
 
         SettingsDialog.__init__(self, parent, _('Connection Settings'),
                                 Gtk.DialogFlags.MODAL, settings, account)
 
 
 class CertificateDialog(SettingsDialog):
-    def __init__(self, account, parent):
+    def __init__(self, account: str, parent: Gtk.Window) -> None:
 
         settings = [
             Setting(SettingKind.FILECHOOSER, _('Client Certificate'),
@@ -990,14 +998,14 @@ class CertificateDialog(SettingsDialog):
 
             Setting(SettingKind.SWITCH, _('Encrypted Certificate'),
                     SettingType.ACCOUNT_CONFIG, 'client_cert_encrypted'),
-            ]
+        ]
 
         SettingsDialog.__init__(self, parent, _('Certificate Settings'),
                                 Gtk.DialogFlags.MODAL, settings, account)
 
 
 class LoginDialog(SettingsDialog):
-    def __init__(self, account, parent):
+    def __init__(self, account: str, parent: Gtk.Window) -> None:
 
         settings = [
             Setting(SettingKind.ENTRY, _('Password'),
@@ -1005,7 +1013,9 @@ class LoginDialog(SettingsDialog):
                     bind='account::savepass'),
 
             Setting(SettingKind.SWITCH, _('Save Password'),
-                    SettingType.ACCOUNT_CONFIG, 'savepass'),
+                    SettingType.ACCOUNT_CONFIG, 'savepass',
+                    enabled_func=(lambda: not app.settings.get('use_keyring')
+                                  or passwords.KEYRING_AVAILABLE)),
 
             Setting(SettingKind.CHANGEPASSWORD, _('Change Password'),
                     SettingType.DIALOG, callback=self.on_password_change,
@@ -1013,17 +1023,17 @@ class LoginDialog(SettingsDialog):
 
             Setting(SettingKind.SWITCH, _('Use GSSAPI'),
                     SettingType.ACCOUNT_CONFIG, 'enable_gssapi'),
-            ]
+        ]
 
         SettingsDialog.__init__(self, parent, _('Login Settings'),
                                 Gtk.DialogFlags.MODAL, settings, account)
 
         self.connect('destroy', self.on_destroy)
 
-    def on_password_change(self, new_password, _data):
+    def on_password_change(self, new_password: str, _data: Any) -> None:
         passwords.save_password(self.account, new_password)
 
-    def on_destroy(self, *args):
+    def on_destroy(self, *args: Any) -> None:
         savepass = app.settings.get_account_setting(self.account, 'savepass')
         if not savepass:
             passwords.delete_password(self.account)
