@@ -12,18 +12,24 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Optional
+from typing import Tuple
+
+from gi.repository import Gtk
 try:
     from gi.repository import Gst
 except Exception:
     pass
 
 
-def create_gtk_widget():
+def create_gtk_widget() -> Optional[Tuple[Gst.Element,
+                                          Gtk.Widget,
+                                          str]]:
     gtkglsink = Gst.ElementFactory.make('gtkglsink', None)
     if gtkglsink is not None:
         glsinkbin = Gst.ElementFactory.make('glsinkbin', None)
         if glsinkbin is None:
-            return None, None, None
+            return None
         glsinkbin.set_property('sink', gtkglsink)
         sink = glsinkbin
         widget = gtkglsink.get_property('widget')
@@ -31,7 +37,7 @@ def create_gtk_widget():
     else:
         sink = Gst.ElementFactory.make('gtksink', None)
         if sink is None:
-            return None, None, None
+            return None
         widget = sink.get_property('widget')
         name = 'gtksink'
     widget.set_visible(True)
