@@ -12,34 +12,54 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
+from typing import cast
+from typing import Any
+from typing import Optional
+from typing import Union
+
 import locale
 from enum import IntEnum
 
 from gi.repository import Gdk
+from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 from gi.repository import GLib
 from gi.repository import Pango
 
+from nbxmpp import JID
 from nbxmpp.errors import is_error
 from nbxmpp.errors import StanzaError
+from nbxmpp.errors import TimeoutStanzaError
 from nbxmpp.errors import CancelledError
+from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import MuclumbusItem
+from nbxmpp.structs import MuclumbusResult
+from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common.const import Direction
 from gajim.common.helpers import validate_jid
 from gajim.common.helpers import to_user_string
-from gajim.common.helpers import get_groupchat_name
 from gajim.common.helpers import get_group_chat_nick
 from gajim.common.i18n import _
 from gajim.common.i18n import get_rfc5646_lang
 from gajim.common.const import AvatarSize
 from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.modules.util import as_task
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
 
+from .menus import get_start_chat_row_menu
+from .chat_filter import ChatFilter
 from .groupchat_info import GroupChatInfoScrolled
 from .groupchat_nick import NickChooser
-from .util import get_builder
+from .builder import get_builder
 from .util import get_icon_name
-from .util import generate_account_badge
+from .util import AccountBadge
+
+ContactT = Union[BareContact, GroupchatContact]
 
 
 class Search(IntEnum):
@@ -48,7 +68,7 @@ class Search(IntEnum):
 
 
 class StartChatDialog(Gtk.ApplicationWindow):
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_name('StartChatDialog')
         self.set_application(app.app)
@@ -58,12 +78,11 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
         self.set_default_size(-1, 600)
         self.ready_to_destroy = False
-        self._parameter_form = None
-        self._keywords = []
+        self._parameter_form: Optional[MuclumbusResult] = None
+        self._keywords: list[str] = []
         self._destroyed = False
         self._search_stopped = False
         self._redirected = False
-        self._source_id = None
 
         self._ui = get_builder('start_chat_dialog.ui')
         self.add(self._ui.stack)
@@ -71,22 +90,25 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._nick_chooser = NickChooser()
         self._ui.join_box.pack_start(self._nick_chooser, True, False, 0)
 
-        self.new_contact_row_visible = False
-        self.new_contact_rows = {}
-        self.new_groupchat_rows = {}
+        # Helper for the case where we don't receive a disco info
+        self._new_chat_row: Optional[ContactRow] = None
+        self._search_is_valid_jid = False
+
+        self.new_contact_rows: dict[str, Optional[ContactRow]] = {}
         self._accounts = app.get_enabled_accounts_with_labels()
 
-        rows = []
+        rows: list[ContactRow] = []
         self._add_accounts()
         self._add_contacts(rows)
         self._add_groupchats(rows)
+        self._add_new_contact_rows(rows)
 
-        self._ui.search_entry.connect('search-changed',
-                                      self._on_search_changed)
-        self._ui.search_entry.connect('next-match',
-                                      self._select_new_match, 'next')
-        self._ui.search_entry.connect('previous-match',
-                                      self._select_new_match, 'prev')
+        self._ui.search_entry.connect(
+            'search-changed', self._on_search_changed)
+        self._ui.search_entry.connect(
+            'next-match', self._select_new_match, Direction.NEXT)
+        self._ui.search_entry.connect(
+            'previous-match', self._select_new_match, Direction.PREV)
         self._ui.search_entry.connect(
             'stop-search', lambda *args: self._ui.search_entry.set_text(''))
 
@@ -104,80 +126,105 @@ class StartChatDialog(Gtk.ApplicationWindow):
 
         self._ui.infobar.set_revealed(app.settings.get('show_help_start_chat'))
 
+        self._current_filter = 'all'
+        self._chat_filter = ChatFilter()
+        self._chat_filter.connect(
+            'filter-changed', self._on_chat_filter_changed)
+        self._ui.filter_bar_revealer.add(self._chat_filter)
+
         self.connect('key-press-event', self._on_key_press)
-        self.connect('destroy', self._destroy)
+        self.connect('destroy', self._on_destroy)
+
+        if rows:
+            self._load_contacts(rows)
 
         self.select_first_row()
         self._ui.connect_signals(self)
         self.show_all()
 
-        if rows:
-            self._load_contacts(rows)
-
-    def set_search_text(self, text):
+    def set_search_text(self, text: str) -> None:
         self._ui.search_entry.set_text(text)
 
-    def _global_search_active(self):
+    def remove_row(self, account: str, jid: str) -> None:
+        for row in cast(list[ContactRow], self._ui.listbox.get_children()):
+            if row.account == account and row.jid == jid:
+                row.destroy()
+                return
+
+    def _global_search_active(self) -> bool:
         return self._ui.global_search_toggle.get_active()
 
-    def _add_accounts(self):
+    def _add_accounts(self) -> None:
         for account in self._accounts:
             self._ui.account_store.append([None, *account])
 
-    def _add_contacts(self, rows):
+    def _add_contacts(self, rows: list[ContactRow]):
         show_account = len(self._accounts) > 1
         for account, _label in self._accounts:
-            self.new_contact_rows[account] = None
-            for jid in app.contacts.get_jid_list(account):
-                contact = app.contacts.get_contact_with_highest_priority(
-                    account, jid)
-                if contact.is_groupchat:
-                    continue
-                rows.append(ContactRow(account, contact, jid,
-                                       contact.get_shown_name(), show_account))
+            client = app.get_client(account)
+            for jid, _data in client.get_module('Roster').iter():
+                contact = client.get_module('Contacts').get_contact(jid)
+                rows.append(ContactRow(account,
+                                       contact,
+                                       jid,
+                                       contact.name,
+                                       show_account))
+            self_contact = client.get_module('Contacts').get_contact(
+                client.get_own_jid().bare)
+            rows.append(ContactRow(account,
+                                   self_contact,
+                                   self_contact.jid,
+                                   _('Note to myself'),
+                                   show_account))
 
-    def _add_groupchats(self, rows):
+    def _add_groupchats(self, rows: list[ContactRow]) -> None:
         show_account = len(self._accounts) > 1
         for account, _label in self._accounts:
-            self.new_groupchat_rows[account] = None
-            con = app.connections[account]
-            bookmarks = con.get_module('Bookmarks').bookmarks
+            client = app.get_client(account)
+            bookmarks = client.get_module('Bookmarks').bookmarks
             for bookmark in bookmarks:
-                jid = str(bookmark.jid)
-                name = get_groupchat_name(con, jid)
-                rows.append(ContactRow(account, None, jid, name, show_account,
+                contact = client.get_module('Contacts').get_contact(
+                    bookmark.jid, groupchat=True)
+                rows.append(ContactRow(account,
+                                       contact,
+                                       bookmark.jid,
+                                       contact.name,
+                                       show_account,
                                        groupchat=True))
 
-    def _load_contacts(self, rows):
-        generator = self._incremental_add(rows)
-        self._source_id = GLib.idle_add(lambda: next(generator, False),
-                                        priority=GLib.PRIORITY_LOW)
+    def _add_new_contact_rows(self, rows: list[ContactRow]) -> None:
+        for account, _ in self._accounts:
+            show_account = len(self._accounts) > 1
+            row = ContactRow(account, None, None, None, show_account)
+            self.new_contact_rows[account] = row
+            rows.append(row)
 
-    def _incremental_add(self, rows):
+    def _load_contacts(self, rows: list[ContactRow]) -> None:
         for row in rows:
             self._ui.listbox.add(row)
-            yield True
 
         self._ui.listbox.set_sort_func(self._sort_func, None)
-        self._source_id = None
 
-    def _on_page_changed(self, stack, _param):
+    def _on_page_changed(self, stack: Gtk.Stack, _param: Any) -> None:
         if stack.get_visible_child_name() == 'account':
             self._ui.account_view.grab_focus()
 
-    def _on_row_activated(self, _listbox, row):
+    def _on_row_activated(self,
+                          _listbox: Gtk.ListBox,
+                          row: ContactRow
+                          ) -> None:
         if self._current_listbox_is(Search.GLOBAL):
             self._select_muc()
         else:
             self._start_new_chat(row)
 
-    def _select_muc(self):
+    def _select_muc(self) -> None:
         if len(self._accounts) > 1:
             self._ui.stack.set_visible_child_name('account')
         else:
             self._on_select_clicked()
 
-    def _on_key_press(self, _widget, event):
+    def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> int:
         is_search = self._ui.stack.get_visible_child_name() == 'search'
         if event.keyval in (Gdk.KEY_Down, Gdk.KEY_Tab):
             if not is_search:
@@ -267,19 +314,34 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._ui.search_entry.grab_focus_without_selecting()
         return Gdk.EVENT_PROPAGATE
 
-    def _on_infobar_response(self, _widget, response):
+    def _on_infobar_response(self,
+                             _widget: Gtk.InfoBar,
+                             response: Gtk.ResponseType
+                             ) -> None:
         if response == Gtk.ResponseType.CLOSE:
             self._ui.infobar.set_revealed(False)
             app.settings.set('show_help_start_chat', False)
 
-    def _start_new_chat(self, row):
-        if row.new:
+    def _on_filter_bar_toggled(self, toggle_button: Gtk.ToggleButton) -> None:
+        active = toggle_button.get_active()
+        self._ui.filter_bar_revealer.set_reveal_child(active)
+
+    def _on_chat_filter_changed(self, _filter: ChatFilter, name: str) -> None:
+        self._current_filter = name
+        self._ui.listbox.invalidate_filter()
+
+    def _start_new_chat(self, row: ContactRow) -> None:
+        if row.is_new:
             try:
                 validate_jid(row.jid)
             except ValueError as error:
-                self._show_error_page(error)
+                self._show_error_page(str(error))
                 return
 
+            self._disco_info(row)
+            return
+
+        jid = JID.from_string(row.jid)
         if row.groupchat:
             if not app.account_is_available(row.account):
                 self._show_error_page(_('You can not join a group chat '
@@ -287,30 +349,87 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 return
 
             self.ready_to_destroy = True
-            if app.interface.show_groupchat(row.account, row.jid):
+            if app.window.chat_exists(row.account, jid):
+                app.window.select_chat(row.account, jid)
+                self.destroy()
                 return
 
             self.ready_to_destroy = False
             self._redirected = False
-            self._disco_muc(row.account, row.jid, request_vcard=row.new)
+            self._disco_muc(row.account, jid, request_vcard=row.is_new)
 
         else:
-            app.interface.new_chat_from_jid(row.account, row.jid)
+            app.window.add_chat(
+                row.account,
+                jid,
+                'contact',
+                select=True,
+                workspace='current')
             self.ready_to_destroy = True
+            self.destroy()
+
+    def _disco_info(self, row: ContactRow) -> None:
+        if not app.account_is_available(row.account):
+            self._show_error_page(_('You are offline.'))
+            return
 
-    def _disco_muc(self, account, jid, request_vcard):
         self._ui.stack.set_visible_child_name('progress')
-        con = app.connections[account]
-        con.get_module('Discovery').disco_muc(
+        client = app.get_client(row.account)
+        client.get_module('Discovery').disco_info(
+            row.jid,
+            callback=self._disco_info_received,
+            user_data=row,
+            timeout=10)
+
+    def _disco_info_received(self, task: Task) -> None:
+        row = cast(ContactRow, task.get_user_data())
+        try:
+            result = cast(DiscoInfo, task.finish())
+        except StanzaError as error:
+            contact_conditions = [
+                'service-unavailable',  # Prosody
+                'subscription-required'  # ejabberd
+            ]
+            if error.condition in contact_conditions:
+                # These error conditions are the result of
+                # querying contacts without subscription
+                row.update_chat_type()
+                self._start_new_chat(row)
+                return
+
+            # Handle other possible errors
+            self._show_error_page(error.get_text())
+            return
+        except TimeoutStanzaError:
+            # We reached the 10s timeout and we cannot
+            # assume which kind contact this is.
+            self._show_error_page(_('This address is not reachable.'))
+            return
+
+        if result.is_muc:
+            row.update_chat_type(groupchat=True)
+        else:
+            row.update_chat_type()
+        self._start_new_chat(row)
+
+    def _on_no_disco_continue(self, _button: Gtk.Button) -> None:
+        assert self._new_chat_row
+        self._new_chat_row.update_chat_type()
+        self._start_new_chat(self._new_chat_row)
+
+    def _disco_muc(self, account: str, jid: JID, request_vcard: bool) -> None:
+        self._ui.stack.set_visible_child_name('progress')
+        client = app.get_client(account)
+        client.get_module('Discovery').disco_muc(
             jid,
             request_vcard=request_vcard,
             allow_redirect=True,
-            callback=self._disco_info_received,
+            callback=self._muc_disco_info_received,
             user_data=account)
 
-    def _disco_info_received(self, task):
+    def _muc_disco_info_received(self, task: Task) -> None:
         try:
-            result = task.finish()
+            result = cast(DiscoInfo, task.finish())
         except StanzaError as error:
             self._set_error(error)
             return
@@ -327,33 +446,36 @@ class StartChatDialog(Gtk.ApplicationWindow):
         else:
             self._set_error_from_code('not-muc-service')
 
-    def _set_error(self, error):
+    def _set_error(self, error: StanzaError) -> None:
         text = MUC_DISCO_ERRORS.get(error.condition, to_user_string(error))
         if error.condition == 'gone':
             reason = error.get_text(get_rfc5646_lang())
             if reason:
-                text = '%s:\n%s' % (text, reason)
+                text = f'{text}:\n{reason}'
         self._show_error_page(text)
 
-    def _set_error_from_code(self, error_code):
+    def _set_error_from_code(self, error_code: str) -> None:
         self._show_error_page(MUC_DISCO_ERRORS[error_code])
 
-    def _show_error_page(self, text):
+    def _show_error_page(self, text: str) -> None:
         self._ui.error_label.set_text(str(text))
         self._ui.stack.set_visible_child_name('error')
 
-    def _on_join_clicked(self, _button=None):
+    def _on_join_clicked(self, *args: Any) -> None:
         account = self._muc_info_box.get_account()
         jid = self._muc_info_box.get_jid()
         nickname = self._nick_chooser.get_text()
-        app.interface.show_or_join_groupchat(
-            account, str(jid), nick=nickname)
-        self.ready_to_destroy = True
+        assert account
+        app.interface.show_add_join_groupchat(
+            account, str(jid), nickname=nickname)
 
-    def _on_back_clicked(self, _button=None):
+        self.ready_to_destroy = True
+        self.destroy()
+
+    def _on_back_clicked(self, *args: Any) -> None:
         self._ui.stack.set_visible_child_name('search')
 
-    def _on_select_clicked(self, *args):
+    def _on_select_clicked(self, *args: Any) -> None:
         model, iter_ = self._ui.account_view.get_selection().get_selected()
         if iter_ is not None:
             account = model[iter_][1]
@@ -362,7 +484,8 @@ class StartChatDialog(Gtk.ApplicationWindow):
         else:
             return
 
-        selected_row = self._global_search_listbox.get_selected_row()
+        selected_row = cast(
+            ResultRow, self._global_search_listbox.get_selected_row())
         if selected_row is None:
             return
 
@@ -374,7 +497,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._redirected = False
         self._disco_muc(account, selected_row.jid, request_vcard=True)
 
-    def _set_listbox(self, listbox):
+    def _set_listbox(self, listbox: Gtk.ListBox):
         if self._current_listbox == listbox:
             return
         viewport = self._ui.scrolledwindow.get_child()
@@ -383,77 +506,73 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._ui.scrolledwindow.add(listbox)
         self._current_listbox = listbox
 
-    def _current_listbox_is(self, box):
+    def _current_listbox_is(self, box: Search) -> bool:
         if self._current_listbox == self._ui.listbox:
             return box == Search.CONTACT
         return box == Search.GLOBAL
 
-    def _on_global_search_toggle(self, button):
+    def _on_global_search_toggle(self, button: Gtk.ToggleButton) -> None:
         self._ui.search_entry.grab_focus()
         image_style_context = button.get_children()[0].get_style_context()
         if button.get_active():
+            self._ui.filter_bar_toggle.set_active(False)
+            self._ui.filter_bar_toggle.set_sensitive(False)
             image_style_context.add_class('selected-color')
             self._set_listbox(self._global_search_listbox)
             if self._ui.search_entry.get_text():
                 self._start_search()
-            self._remove_new_jid_row()
             self._ui.listbox.invalidate_filter()
         else:
+            self._ui.filter_bar_toggle.set_sensitive(True)
             self._ui.search_entry.set_text('')
             image_style_context.remove_class('selected-color')
             self._set_listbox(self._ui.listbox)
             self._global_search_listbox.remove_all()
 
-    def _on_search_changed(self, entry):
+    def _on_search_changed(self, search_entry: Gtk.SearchEntry) -> None:
+        self._show_search_entry_error(False)
+        self._search_is_valid_jid = False
+
         if self._global_search_active():
             return
 
-        search_text = entry.get_text()
+        search_text = search_entry.get_text()
         if '@' in search_text:
-            self._add_new_jid_row()
-            self._update_new_jid_rows(search_text)
-        else:
-            self._remove_new_jid_row()
+            try:
+                validate_jid(search_text)
+            except ValueError:
+                self._show_search_entry_error(True)
+            else:
+                self._update_new_contact_rows(search_text)
+                self._search_is_valid_jid = True
+
         self._ui.listbox.invalidate_filter()
 
-    def _add_new_jid_row(self):
-        if self.new_contact_row_visible:
-            return
-        for account in self.new_contact_rows:
-            show_account = len(self._accounts) > 1
-            row = ContactRow(account, None, '', None, show_account)
-            self.new_contact_rows[account] = row
-            group_row = ContactRow(account, None, '', None, show_account,
-                                   groupchat=True)
-            self.new_groupchat_rows[account] = group_row
-            self._ui.listbox.add(row)
-            self._ui.listbox.add(group_row)
-            row.get_parent().show_all()
-        self.new_contact_row_visible = True
+    def _show_search_entry_error(self, state: bool):
+        icon_name = 'dialog-warning-symbolic' if state else None
+        self._ui.search_entry.set_icon_from_icon_name(
+            Gtk.EntryIconPosition.SECONDARY,
+            icon_name)
+        self._ui.search_entry.set_icon_tooltip_text(
+            Gtk.EntryIconPosition.SECONDARY,
+            _('Invalid Address'))
 
-    def _remove_new_jid_row(self):
-        if not self.new_contact_row_visible:
-            return
-        for account in self.new_contact_rows:
-            self._ui.listbox.remove(
-                self.new_contact_rows[account])
-            self._ui.listbox.remove(
-                self.new_groupchat_rows[account])
-        self.new_contact_row_visible = False
+    def _update_new_contact_rows(self, search_text: str) -> None:
+        for row in self.new_contact_rows.values():
+            if row is not None:
+                row.update_jid(JID.from_string(search_text))
 
-    def _update_new_jid_rows(self, search_text):
-        for account in self.new_contact_rows:
-            self.new_contact_rows[account].update_jid(search_text)
-            self.new_groupchat_rows[account].update_jid(search_text)
-
-    def _select_new_match(self, _entry, direction):
+    def _select_new_match(self,
+                          _entry: Gtk.Entry,
+                          direction: Direction
+                          ) -> None:
         selected_row = self._ui.listbox.get_selected_row()
         if selected_row is None:
             return
 
         index = selected_row.get_index()
 
-        if direction == 'next':
+        if direction == Direction.NEXT:
             index += 1
         else:
             index -= 1
@@ -466,39 +585,50 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 self._ui.listbox.select_row(new_selected_row)
                 new_selected_row.grab_focus()
                 return
-            if direction == 'next':
+            if direction == Direction.NEXT:
                 index += 1
             else:
                 index -= 1
 
-    def select_first_row(self):
+    def select_first_row(self) -> None:
         first_row = self._ui.listbox.get_row_at_y(0)
         self._ui.listbox.select_row(first_row)
 
-    def _scroll_to_first_row(self):
+    def _scroll_to_first_row(self) -> None:
         self._ui.scrolledwindow.get_vadjustment().set_value(0)
 
-    def _filter_func(self, row, _user_data):
+    def _filter_func(self, row: ContactRow, _user_data: Any) -> bool:
+        if row.contact is None:
+            # new contact row
+            return self._search_is_valid_jid
+
         search_text = self._ui.search_entry.get_text().lower()
         search_text_list = search_text.split()
         row_text = row.get_search_text().lower()
+
+        if self._current_filter == 'chats' and row.groupchat:
+            return False
+
+        if self._current_filter == 'group_chats' and not row.groupchat:
+            return False
+
         for text in search_text_list:
             if text not in row_text:
                 GLib.timeout_add(50, self.select_first_row)
-                return None
+                return False
         GLib.timeout_add(50, self.select_first_row)
         return True
 
     @staticmethod
-    def _sort_func(row1, row2, _user_data):
+    def _sort_func(row1: ContactRow, row2: ContactRow, _user_data: Any) -> int:
         name1 = row1.get_search_text()
         name2 = row2.get_search_text()
         account1 = row1.account
         account2 = row2.account
         is_groupchat1 = row1.groupchat
         is_groupchat2 = row2.groupchat
-        new1 = row1.new
-        new2 = row2.new
+        new1 = row1.is_new
+        new2 = row2.is_new
 
         result = locale.strcoll(account1.lower(), account2.lower())
         if result != 0:
@@ -512,27 +642,27 @@ class StartChatDialog(Gtk.ApplicationWindow):
 
         return locale.strcoll(name1.lower(), name2.lower())
 
-    def _start_search(self):
+    def _start_search(self) -> None:
         self._search_stopped = False
         accounts = app.get_connected_accounts()
         if not accounts:
             return
-        con = app.connections[accounts[0]].connection
+        client = app.connections[accounts[0]].connection
 
         text = self._ui.search_entry.get_text().strip()
         self._global_search_listbox.start_search()
 
         if app.settings.get('muclumbus_api_pref') == 'http':
-            self._start_http_search(con, text)
+            self._start_http_search(client, text)
         else:
-            self._start_iq_search(con, text)
+            self._start_iq_search(client, text)
 
     @as_task
-    def _start_iq_search(self, con, text):
+    def _start_iq_search(self, client, text):
         _task = yield
 
         if self._parameter_form is None:
-            result = yield con.get_module('Muclumbus').request_parameters(
+            result = yield client.get_module('Muclumbus').request_parameters(
                 app.settings.get('muclumbus_api_jid'))
 
             self._process_search_result(result, parameters=True)
@@ -542,14 +672,14 @@ class StartChatDialog(Gtk.ApplicationWindow):
 
         self._parameter_form.vars['q'].value = text
 
-        result = yield con.get_module('Muclumbus').set_search(
+        result = yield client.get_module('Muclumbus').set_search(
             app.settings.get('muclumbus_api_jid'),
             self._parameter_form)
 
         self._process_search_result(result)
 
         while not result.end:
-            result = yield con.get_module('Muclumbus').set_search(
+            result = yield client.get_module('Muclumbus').set_search(
                 app.settings.get('muclumbus_api_jid'),
                 self._parameter_form,
                 items_per_page=result.max,
@@ -560,18 +690,18 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._global_search_listbox.end_search()
 
     @as_task
-    def _start_http_search(self, con, text):
+    def _start_http_search(self, client, text):
         _task = yield
 
         self._keywords = text.split(' ')
-        result = yield con.get_module('Muclumbus').set_http_search(
+        result = yield client.get_module('Muclumbus').set_http_search(
             app.settings.get('muclumbus_api_http_uri'),
             self._keywords)
 
         self._process_search_result(result)
 
         while not result.end:
-            result = yield con.get_module('Muclumbus').set_http_search(
+            result = yield client.get_module('Muclumbus').set_http_search(
                 app.settings.get('muclumbus_api_http_uri'),
                 self._keywords,
                 after=result.last)
@@ -580,11 +710,15 @@ class StartChatDialog(Gtk.ApplicationWindow):
 
         self._global_search_listbox.end_search()
 
-    def _process_search_result(self, result, parameters=False):
+    def _process_search_result(self,
+                               result: MuclumbusResult,
+                               parameters: bool = False
+                               ) -> None:
         if self._search_stopped:
             raise CancelledError()
 
         if is_error(result):
+            assert isinstance(result, StanzaError)
             self._global_search_listbox.remove_progress()
             self._show_error_page(to_user_string(result))
             raise result
@@ -595,16 +729,23 @@ class StartChatDialog(Gtk.ApplicationWindow):
         for item in result.items:
             self._global_search_listbox.add(ResultRow(item))
 
-    def _destroy(self, *args):
-        if self._source_id is not None:
-            GLib.source_remove(self._source_id)
+    def _on_destroy(self, *args: Any) -> None:
+        self._ui.listbox.set_filter_func(None)
+        self._ui.listbox.destroy()
         self._destroyed = True
         app.cancel_tasks(self)
+        app.check_finalize(self)
 
 
 class ContactRow(Gtk.ListBoxRow):
-    def __init__(self, account, contact, jid, name, show_account,
-                 groupchat=False):
+    def __init__(self,
+                 account: str,
+                 contact: Optional[ContactT],
+                 jid: Optional[JID],
+                 name: Optional[str],
+                 show_account: bool,
+                 groupchat: bool = False
+                 ) -> None:
         Gtk.ListBoxRow.__init__(self)
         self.get_style_context().add_class('start-chat-row')
         self.account = account
@@ -614,15 +755,13 @@ class ContactRow(Gtk.ListBoxRow):
         self.contact = contact
         self.name = name
         self.groupchat = groupchat
-        self.new = jid == ''
-
-        show = contact.show if contact else 'offline'
+        self.is_new = bool(jid is None)
 
         grid = Gtk.Grid()
         grid.set_column_spacing(12)
         grid.set_size_request(260, -1)
 
-        image = self._get_avatar_image(account, jid, show)
+        image = self._get_avatar_image(contact)
         image.set_size_request(AvatarSize.CHAT, AvatarSize.CHAT)
         grid.add(image)
 
@@ -630,10 +769,7 @@ class ContactRow(Gtk.ListBoxRow):
         box.set_hexpand(True)
 
         if self.name is None:
-            if self.groupchat:
-                self.name = _('Join Group Chat')
-            else:
-                self.name = _('Start Chat')
+            self.name = _('Start New Chat')
 
         self.name_label = Gtk.Label(label=self.name)
         self.name_label.set_ellipsize(Pango.EllipsizeMode.END)
@@ -644,17 +780,15 @@ class ContactRow(Gtk.ListBoxRow):
         name_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
         name_box.add(self.name_label)
         if show_account:
-            account_badge = generate_account_badge(account)
-            account_badge.set_tooltip_text(
-                _('Account: %s') % self.account_label)
+            account_badge = AccountBadge(account)
             account_badge.set_halign(Gtk.Align.END)
             account_badge.set_valign(Gtk.Align.START)
             account_badge.set_hexpand(True)
             name_box.add(account_badge)
         box.add(name_box)
 
-        self.jid_label = Gtk.Label(label=jid)
-        self.jid_label.set_tooltip_text(jid)
+        self.jid_label = Gtk.Label(label=str(jid))
+        self.jid_label.set_tooltip_text(str(jid))
         self.jid_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.jid_label.set_xalign(0)
         self.jid_label.set_width_chars(22)
@@ -664,48 +798,73 @@ class ContactRow(Gtk.ListBoxRow):
 
         grid.add(box)
 
-        self.add(grid)
+        eventbox = Gtk.EventBox()
+        eventbox.connect('button-press-event', self._popup_menu)
+        eventbox.add(grid)
+        self.add(eventbox)
         self.show_all()
 
-    def _get_avatar_image(self, account, jid, show):
-        if self.new:
+    def _popup_menu(self,
+                    _widget: Gtk.EventBox,
+                    event: Gdk.EventButton
+                    ) -> None:
+        if not self.groupchat:
+            return
+
+        if event.button != 3:  # right click
+            return
+
+        menu = get_start_chat_row_menu(self.account, self.jid)
+
+        rectangle = Gdk.Rectangle()
+        rectangle.x = int(event.x)
+        rectangle.y = int(event.y)
+        rectangle.width = rectangle.height = 1
+
+        popover = Gtk.Popover.new_from_model(self, menu)
+        popover.set_relative_to(self)
+        popover.set_position(Gtk.PositionType.RIGHT)
+        popover.set_pointing_to(rectangle)
+        popover.popup()
+
+    def _get_avatar_image(self, contact: ContactT) -> Gtk.Image:
+        if self.is_new:
             icon_name = 'avatar-default'
             if self.groupchat:
                 icon_name = get_icon_name('muc-inactive')
             return Gtk.Image.new_from_icon_name(icon_name, Gtk.IconSize.DND)
 
         scale = self.get_scale_factor()
-        if self.groupchat:
-            surface = app.interface.avatar_storage.get_muc_surface(
-                account, jid, AvatarSize.CHAT, scale)
-            return Gtk.Image.new_from_surface(surface)
+        surface = contact.get_avatar(AvatarSize.CHAT, scale)
+        assert not isinstance(surface, GdkPixbuf.Pixbuf)
+        return Gtk.Image.new_from_surface(surface)
 
-        avatar = app.contacts.get_avatar(
-            account, jid, AvatarSize.CHAT, scale, show)
-        return Gtk.Image.new_from_surface(avatar)
-
-    def update_jid(self, jid):
+    def update_jid(self, jid: JID) -> None:
         self.jid = jid
-        self.jid_label.set_text(jid)
+        self.jid_label.set_text(str(jid))
 
-    def get_search_text(self):
+    def update_chat_type(self, groupchat: bool = False) -> None:
+        self.is_new = False
+        self.groupchat = groupchat
+
+    def get_search_text(self) -> str:
         if self.contact is None and not self.groupchat:
-            return self.jid
+            return str(self.jid)
         if self.show_account:
-            return '%s %s %s' % (self.name, self.jid, self.account_label)
-        return '%s %s' % (self.name, self.jid)
+            return f'{self.name} {self.jid} {self.account_label}'
+        return f'{self.name} {self.jid}'
 
 
 class GlobalSearch(Gtk.ListBox):
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.ListBox.__init__(self)
         self.set_has_tooltip(True)
         self.set_activate_on_single_click(False)
-        self._progress = None
+        self._progress: Optional[ProgressRow] = None
         self._add_placeholder()
         self.show_all()
 
-    def _add_placeholder(self):
+    def _add_placeholder(self) -> None:
         placeholder = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
         placeholder.set_halign(Gtk.Align.CENTER)
         placeholder.set_valign(Gtk.Align.CENTER)
@@ -722,39 +881,41 @@ class GlobalSearch(Gtk.ListBox):
         placeholder.show_all()
         self.set_placeholder(placeholder)
 
-    def remove_all(self):
-        def remove(row):
-            self.remove(row)
+    def remove_all(self) -> None:
+        def remove(row: Gtk.ListBoxRow) -> None:
             row.destroy()
         self.foreach(remove)
 
-    def remove_progress(self):
+    def remove_progress(self) -> None:
+        assert self._progress
         self.remove(self._progress)
         self._progress.destroy()
 
-    def start_search(self):
+    def start_search(self) -> None:
         self._progress = ProgressRow()
         super().add(self._progress)
 
-    def end_search(self):
+    def end_search(self) -> None:
+        assert self._progress
         self._progress.stop()
 
-    def add(self, row):
+    def add(self, row: ResultRow) -> None:
         super().add(row)
         if self.get_selected_row() is None:
-            row = self.get_row_at_index(1)
-            if row is not None:
-                self.select_row(row)
-                row.grab_focus()
+            row_ = self.get_row_at_index(1)
+            if row_ is not None:
+                self.select_row(row_)
+                row_.grab_focus()
+        assert self._progress
         self._progress.update()
 
-    def _select(self, direction):
+    def _select(self, direction: Direction) -> None:
         selected_row = self.get_selected_row()
         if selected_row is None:
             return
 
         index = selected_row.get_index()
-        if direction == 'next':
+        if direction == Direction.NEXT:
             index += 1
         else:
             index -= 1
@@ -766,20 +927,20 @@ class GlobalSearch(Gtk.ListBox):
         self.select_row(new_selected_row)
         new_selected_row.grab_focus()
 
-    def select_next(self):
-        self._select('next')
+    def select_next(self) -> None:
+        self._select(Direction.NEXT)
 
-    def select_prev(self):
-        self._select('prev')
+    def select_prev(self) -> None:
+        self._select(Direction.PREV)
 
 
 class ResultRow(Gtk.ListBoxRow):
-    def __init__(self, item):
+    def __init__(self, item: MuclumbusItem) -> None:
         Gtk.ListBoxRow.__init__(self)
         self.set_activatable(True)
         self.get_style_context().add_class('start-chat-row')
-        self.new = False
-        self.jid = item.jid
+        self.is_new = False
+        self.jid = JID.from_string(item.jid)
         self.groupchat = True
 
         name_label = Gtk.Label(label=item.name)
@@ -800,7 +961,7 @@ class ResultRow(Gtk.ListBoxRow):
 
 
 class ProgressRow(Gtk.ListBoxRow):
-    def __init__(self):
+    def __init__(self) -> None:
         Gtk.ListBoxRow.__init__(self)
         self.set_selectable(False)
         self.set_activatable(False)
@@ -824,11 +985,11 @@ class ProgressRow(Gtk.ListBoxRow):
         self.add(box)
         self.show_all()
 
-    def update(self):
+    def update(self) -> None:
         self._count += 1
         self._count_label.set_text(self._text % self._count)
 
-    def stop(self):
+    def stop(self) -> None:
         self._spinner.stop()
         self._spinner.hide()
         self._finished_image.show()
