@@ -78,6 +78,14 @@ function create_root {
 function install_deps {
     build_pacman --noconfirm -S mingw-w64-"${ARCH}"-python \
         mingw-w64-"${ARCH}"-python-gobject \
+        mingw-w64-"${ARCH}"-gtksourceview4 \
+        mingw-w64-"${ARCH}"-gstreamer \
+        mingw-w64-"${ARCH}"-gst-plugins-base \
+        mingw-w64-"${ARCH}"-gst-plugins-good \
+        mingw-w64-"${ARCH}"-gst-libav \
+        mingw-w64-"${ARCH}"-gst-python \
+        mingw-w64-"${ARCH}"-farstream \
+        mingw-w64-"${ARCH}"-libnice \
         mingw-w64-"${ARCH}"-python-pip \
         mingw-w64-"${ARCH}"-adwaita-icon-theme \
         mingw-w64-"${ARCH}"-libwebp \
@@ -98,13 +106,13 @@ function install_deps {
         build_pip install precis-i18n
 
     PIP_REQUIREMENTS="\
-git+https://dev.gajim.org/gajim/python-nbxmpp.git@nbxmpp_2.0
-git+https://dev.gajim.org/lovetox/pybonjour-python3.git
+git+https://dev.gajim.org/gajim/python-nbxmpp.git
 keyring
 python-gnupg
 python-axolotl
 qrcode
 css_parser
+sentry-sdk
 "
 
     build_pip install $(echo "$PIP_REQUIREMENTS" | tr ["\\n"] [" "])
@@ -140,18 +148,9 @@ function install_gajim {
     build_python "${MISC}"/create-launcher.py \
         "${QL_VERSION}" "${MINGW_ROOT}"/bin
 
-    # Install plugin installer
-    curl -o "${BUILD_ROOT}"/plugin_installer.zip https://ftp.gajim.org/plugins_1.3_zip/plugin_installer.zip
-    mkdir "${PACKAGE_DIR}"/gajim/data/plugins
-    7z x -o"${PACKAGE_DIR}"/gajim/data/plugins "${BUILD_ROOT}"/plugin_installer.zip
-
-    # Install preview plugin
-    curl -o "${BUILD_ROOT}"/url_image_preview.zip https://ftp.gajim.org/plugins_1.3_zip/url_image_preview.zip
-    7z x -o"${PACKAGE_DIR}"/gajim/data/plugins "${BUILD_ROOT}"/url_image_preview.zip
-
     # Install omemo plugin
-    curl -o "${BUILD_ROOT}"/omemo.zip https://ftp.gajim.org/plugins_1.3_zip/omemo.zip
-    7z x -o"${PACKAGE_DIR}"/gajim/data/plugins "${BUILD_ROOT}"/omemo.zip
+    curl -o "${BUILD_ROOT}"/omemo.zip https://ftp.gajim.org/plugins/master/omemo/omemo_2.8.14.zip
+    7z x -o"${PACKAGE_DIR}"/gajim/data/plugins "${BUILD_ROOT}"/omemo_2.8.14.zip
 
     # Install language dicts
     curl -o "${BUILD_ROOT}"/speller_dicts.zip https://gajim.org/downloads/snap/win/build/speller_dicts.zip
@@ -187,16 +186,16 @@ function cleanup_install {
         fi
     done
 
-    KEEP="gajim|gajim-debug|python3|gajim-history-manager|gdbus|gspawn-win32-helper|gspawn-win64-helper"
+    KEEP="gajim|gajim-debug|python3|gdbus|gspawn-win32-helper|gspawn-win64-helper"
 
     echo "deleting .exe files"
     find "${MINGW_ROOT}" -regextype "posix-extended" -name "*.exe" -and ! \
         -iregex ".*/(${KEEP})\.exe" -exec rm -f {} \; -print
 
     rm -Rf "${MINGW_ROOT}"/libexec
-    rm -Rf "${MINGW_ROOT}"/share/gtk-doc
     rm -Rf "${MINGW_ROOT}"/include
     rm -Rf "${MINGW_ROOT}"/var
+    rm -Rf "${MINGW_ROOT}"/share/gtk-doc
     rm -Rf "${MINGW_ROOT}"/share/zsh
     rm -Rf "${MINGW_ROOT}"/share/pixmaps
     rm -Rf "${MINGW_ROOT}"/share/gnome-shell
@@ -220,13 +219,13 @@ function cleanup_install {
     rm -Rf "${MINGW_ROOT}"/share/gdb
     rm -Rf "${MINGW_ROOT}"/share/libcaca
     rm -Rf "${MINGW_ROOT}"/share/gettext
-    rm -Rf "${MINGW_ROOT}"/share/gst-plugins-base
     rm -Rf "${MINGW_ROOT}"/share/gtk-3.0
     rm -Rf "${MINGW_ROOT}"/share/nghttp2
     rm -Rf "${MINGW_ROOT}"/share/fontconfig
     rm -Rf "${MINGW_ROOT}"/share/gettext-*
-    rm -Rf "${MINGW_ROOT}"/share/gstreamer-1.0
     rm -Rf "${MINGW_ROOT}"/share/terminfo
+    rm -Rf "${MINGW_ROOT}"/share/OGRE
+    rm -Rf "${MINGW_ROOT}"/share/opencv4
 
     find "${MINGW_ROOT}"/share/glib-2.0 -type f ! \
         -name "*.compiled" -exec rm -f {} \;
@@ -244,25 +243,29 @@ function cleanup_install {
     rm -Rf "${MINGW_ROOT}"/lib/installed-tests
     rm -Rf "${MINGW_ROOT}"/lib/tabset
     rm -Rf "${MINGW_ROOT}"/lib/libthai
+    rm -Rf "${MINGW_ROOT}"/lib/OGRE
 
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstvpx.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstdaala.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstdvdread.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstopenal.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstopenexr.dll
-    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstopenh264.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstresindvd.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstassrender.dll
-    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstx265.dll
-    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstwebp.dll
-    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstopengl.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstmxf.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstfaac.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstschro.dll
+    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstcacasink.dll
 
     rm -f "${MINGW_ROOT}"/bin/libharfbuzz-icu-0.dll
+    rm -f "${MINGW_ROOT}"/bin/libopencv_wechat_qrcode455.dll
+    rm -f "${MINGW_ROOT}"/bin/OgreMain.dll
+    rm -f "${MINGW_ROOT}"/bin/OgreOverlay.dll
+    rm -f "${MINGW_ROOT}"/bin/OgreRTShaderSystem.dll
+    rm -f "${MINGW_ROOT}"/bin/OgreBites.dll
+    rm -f "${MINGW_ROOT}"/bin/xvidcore.dll
+
     rm -f "${MINGW_ROOT}"/lib/"${PYTHON_ID}".*/lib-dynload/_tkinter*
-    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstcacasink.dll
 
     rm -Rf "${MINGW_ROOT}"/lib/python2.*
 
