@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -8,20 +7,20 @@
 
 """
 Deletes unneeded DLLs and checks DLL dependencies.
-Execute with the build python, will figure out the rest.
 """
 
 import subprocess
 import os
 import sys
+from typing import Optional
 
 import gi
 gi.require_version("GIRepository", "2.0")
 from gi.repository import GIRepository
 
 
-def get_required_by_typelibs():
-    deps = set()
+def get_required_by_typelibs() -> set[str]:
+    deps: set[str] = set()
     repo = GIRepository.Repository()
     for tl in os.listdir(repo.get_search_path()[0]):
         namespace, version = os.path.splitext(tl)[0].split("-", 1)
@@ -33,7 +32,8 @@ def get_required_by_typelibs():
 
 
 EXTENSIONS = [".exe", ".pyd", ".dll"]
-SYSTEM_LIBS = ['advapi32.dll',
+SYSTEM_LIBS = [
+    "advapi32.dll",
     "cabinet.dll", "comctl32.dll", "comdlg32.dll", "crypt32.dll", "d3d9.dll",
     "dnsapi.dll", "dsound.dll", "dwmapi.dll", "gdi32.dll", "imm32.dll",
     "iphlpapi.dll", "kernel32.dll", "ksuser.dll", "msi.dll", "msimg32.dll",
@@ -44,8 +44,8 @@ SYSTEM_LIBS = ['advapi32.dll',
 ]
 
 
-def get_dependencies(filename):
-    deps = []
+def get_dependencies(filename: str) -> list[str]:
+    deps: list[str] = []
     try:
         data = subprocess.getoutput("objdump -p %s" % filename)
     except Exception as error:
@@ -59,7 +59,7 @@ def get_dependencies(filename):
     return deps
 
 
-def find_lib(root, name):
+def find_lib(root: str, name: str) -> Optional[str]:
     search_path = os.path.join(root, "bin")
     if os.path.exists(os.path.join(search_path, name)):
         return os.path.join(search_path, name)
@@ -67,10 +67,10 @@ def find_lib(root, name):
         return name
 
 
-def get_things_to_delete(root):
-    all_libs = set()
-    needed = set()
-    for base, dirs, files in os.walk(root):
+def get_things_to_delete(root: str) -> list[str]:
+    all_libs: set[str] = set()
+    needed: set[str] = set()
+    for base, _, files in os.walk(root):
         for f in files:
             path = os.path.join(base, f)
             if os.path.splitext(path)[-1].lower() in EXTENSIONS:
@@ -78,29 +78,36 @@ def get_things_to_delete(root):
                 for lib in get_dependencies(path):
                     all_libs.add(lib)
                     needed.add(lib)
-                    if not find_lib(root, lib):
+                    if find_lib(root, lib) is None:
                         print("MISSING:", path, lib)
 
     for lib in get_required_by_typelibs():
         needed.add(lib)
-        if not find_lib(root, lib):
-            print("MISSING:", path, lib)
+        if find_lib(root, lib) is None:
+            print("MISSING:", lib)
 
-    # get rid of things not in the search path,
-    # maybe loaded through other means?
-    not_needed = filter(
-        lambda l: find_lib(root, l) and \
-            os.path.splitext(l)[-1].lower() != ".exe", all_libs - needed)
+    result: list[str] = []
+    libs = all_libs - needed
+    for lib in libs:
+        _, ext = os.path.splitext(lib)
+        if ext == ".exe":
+            continue
 
-    return [find_lib(root, l) for l in not_needed]
+        name = find_lib(root, lib)
+        if name is None:
+            continue
+
+        result.append(name)
+
+    return result
 
 
-def main():
+def main() -> None:
     libs = get_things_to_delete(sys.prefix)
     while libs:
-        for l in libs:
-            print("DELETE:", l)
-            os.unlink(l)
+        for lib in libs:
+            print("DELETE:", lib)
+            os.unlink(lib)
         libs = get_things_to_delete(sys.prefix)
 
 
